function ModelParams = alg_SVMlightMEX_learn(AlgParams, Inputs, Labels)
% function ModelParams = alg_SVMlightMEX_learn(AlgParams, Inputs, Labels)
%
% AlgParams is an SVMlight parameter string

%ModelParams = svm_Learn(Inputs, Labels .* 2 - 1, AlgParams.KernelType, AlgParams.KernelParams, AlgParams.C, AlgParams.Epsilon);

% Kernel
switch AlgParams.KernelType
    case 'linear'
        kernel = 0;
    case 'poly'
        kernel = 1;
    case 'rbf'
        kernel = 2;
    otherwise
        error(['Unknown SVMlight kernel: ', AlgParams.KernelType]);
end

% CostFactor = #neg / #pos
costfactor = (size(Labels,1) / sum(Labels)) - 1;

% set up svm
%opts = svmlopt('Kernel', kernel, ...
%    'CostFactor', costfactor, ...
%    'C', AlgParams.C, ...
%   'EpsTermin', AlgParams.Epsilon, ...
%    'KernelParam', AlgParams.KernelParam, ...
%    'ExecPath', './');

% build parameter string
ParamStr = sprintf('-t %i', kernel);

ParamStr = [ParamStr, sprintf(' -j %.10g', costfactor)];

if isfield(AlgParams, 'C') && ~isempty(AlgParams.C),
  ParamStr = [ParamStr, sprintf(' -c %.10g', AlgParams.C)];
end

if isfield(AlgParams, 'KernelParam') && ~isempty(AlgParams.KernelParam),
  if kernel == 1, % poly
    ParamStr = [ParamStr, sprintf(' -d %.10g', AlgParams.KernelParam)];
  elseif kernel == 2, % rbf
    ParamStr = [ParamStr, sprintf(' -g %.10g', AlgParams.KernelParam)];
  else
    error(['KernelParam specified for invalid kernel type']);
  end
end

ParamStr = [ParamStr, sprintf(' -e %.10g', AlgParams.Epsilon)];

ModelParams = struct;

% store for reference
ModelParams.ParamStr = ParamStr;

if isequal(AlgParams.NormalizeInputs, 'yes')
    ModelParams.NormalizeInputs = 'yes';
    disp('WARNING: alg_SVMlightMEX_learn normalizing inputs...');
    [NormalizedInputs, ModelParams.ScalingMean, ModelParams.ScalingStd] = normalizeInputs(Inputs);
    disp('alg_SVMlightMEX_learn normalizing inputs done.');

    ModelParams.SVM = svmlearn(NormalizedInputs, Labels .* 2 - 1, ParamStr);
else
    ModelParams.NormalizeInputs = 'no';
    
    ModelParams.SVM = svmlearn(Inputs, Labels .* 2 - 1, ParamStr);
end

return
