function ModelParams = alg_SVMlight_learn(AlgParams, Inputs, Labels)
% function ModelParams = alg_SVMlight_learn(AlgParams, Inputs, Labels)

%ModelParams = svm_Learn(Inputs, Labels .* 2 - 1, AlgParams.KernelType, AlgParams.KernelParams, AlgParams.C, AlgParams.Epsilon);

% Kernel
switch AlgParams.KernelType
    case 'linear'
        kernel = 0;
    case 'poly'
        kernel = 1;
    case 'rbf'
        kernel = 2;
    otherwise
        error(['Unknown SVMlight kernel: ', AlgParams.KernelType]);
end

% CostFactor = #neg / #pos
costfactor = (size(Labels,1) / sum(Labels)) - 1;

% set up svm
opts = svmlopt('Kernel', kernel, ...
    'CostFactor', costfactor, ...
    'C', AlgParams.C, ...
    'EpsTermin', AlgParams.Epsilon, ...
    'KernelParam', AlgParams.KernelParam, ...
    'ExecPath', './');
ModelParams = struct;
ModelParams.SVM = svml('', opts);
%ModelParams.SVM.kkttol = AlgParams.kkttol;
%ModelParams.SVM.alphatol = AlgParams.alphatol;

if isequal(AlgParams.NormalizeInputs, 'yes')
    ModelParams.NormalizeInputs = 'yes';
    disp('WARNING: alg_SVMlight_learn normalizing inputs...');
    [NormalizedInputs, ModelParams.ScalingMean, ModelParams.ScalingStd] = normalizeInputs(Inputs);
    disp('alg_SVMlight_learn normalizing inputs done.');

    ModelParams.SVM = svmltrain(ModelParams.SVM, NormalizedInputs, Labels .* 2 - 1);
else
    ModelParams.NormalizeInputs = 'no';
    
    ModelParams.SVM = svmltrain(ModelParams.SVM, Inputs, Labels .* 2 - 1);
end

return
