function AlgParams = alg_kNN_selectParams(AlgParams, Inputs, Labels, verbose)
% function [AlgParams] = alg_kNN_selectParams(AlgParams, Inputs, Labels, [verbose=1])
%
% Selects k for kNN using validation
if nargin < 4, verbose = 1; end

N = size(Inputs, 1);
D = size(Inputs, 2);

minK = 1;
maxK = 9;
stepK = 2;

ParamSelectionInfo = struct;

K = minK:stepK:maxK;
ParamSelectionInfo.kRange = [minK, stepK, maxK];

if verbose >= 1, 
    disp(['Search for kNN parameter...']);
end
[K_min, E_min, E_valid] = iterate(K, AlgParams, Inputs, Labels, verbose);
ParamSelectionInfo.Errors = E_valid;

%        % finer search -- 5cv
%        numfolds = 5;
%        indices = (1:N)'; % all data
%        Folds = sampleFoldsByClass(indices, Labels(indices), numfolds);
%        if verbose >= 1, 
%            disp(['Refining search around C=', num2str(C_min), ' ...']);
%        end        
%        C = C_min .* (2.^(-2:0.25:2));
%        ParamSelectionInfo.Fine_CRange = C;
%        % E_valid(find(C == C_min)) = E_min; now we change data size
%        [C_min, E_min, E_valid] = iterate_linear(C, AlgParams, Inputs, Labels, Folds, verbose);
%        ParamSelectionInfo.Fine_Errors = E_valid;

AlgParams.k = K_min;
AlgParams.ParamSelectionInfo = ParamSelectionInfo;
if verbose >= 1, 
    disp(['Selected k=', num2str(K_min), '.']);
end

return
%-------------------------------------------------------------------------
% function [C_min, E_min] = iterate_linear(C, E_valid, AlgParams, Inputs, Labels, I_train, I_valid, verbose)
% for i=1:length(C)
%     if E_valid(i) >= 0, continue; end;
%     AlgParams.C = C(i);
%     ModelParams = alg_SVMlight_learn(AlgParams, Inputs(I_train,:), Labels(I_train));
%     Outputs = alg_SVMlight_eval(ModelParams, Inputs(I_valid,:));
%     %E_valid(i) = sum(((Outputs - 0.5) .* (Labels(I_valid) - 0.5)) <= 0);
%     E_valid(i) = - sum(Labels(I_valid) .* log(Outputs) + (1 - Labels(I_valid)) .* log(1 - Outputs));
%     if verbose >= 1
%         disp(['Error for C', num2str(C(i)), ' = ', num2str(E_valid(i))]);
%     end
% end
function [K_min, E_min, E_valid] = iterate(K, AlgParams, Inputs, Labels, verbose)
E_valid = zeros(length(K), 1) .* Inf;
for i_K = 1:length(K)
    %if E_valid(i) >= 0, continue; end;

    if K(i_K) >= length(Labels), continue; end;
        
    AlgParams.k = K(i_K);

    ModelParams = alg_kNN_learn(AlgParams, Inputs, Labels);
    ModelParams.LeaveOneOutNow = 1;
    Outputs = alg_kNN_eval(ModelParams, Inputs);
    ModelParams.LeaveOneOutNow = 0;
    %E_valid(i_K) = - sum(Labels .* log(Outputs) + (1 - Labels) .* log(1 - Outputs));
    
    Stats = GetConfusionStats(GetConfusion(Labels, Outputs));
    E_valid(i_K) = - Stats.FMeasure;
    
    %E_valid(i_K) = sum(Labels ~= Outputs) ./ size(Labels,1);

    if verbose >= 1
        disp(['Error for k=', num2str(K(i_K)), ': ', num2str(E_valid(i_K))]);
    end
end
E_min = min(E_valid);
I_min = find(E_valid == E_min);
K_min = K(I_min(1));
return
