function Stats = GetConfusionStats(C, N1N2adjustment)
% C is a 2x2 confusion matrix
% N1N2adjustment is optional #neg(N1)/#neg(N2) ratio for Adj.Prec.

Stats = struct;
all = permute(C(1,1,:) + C(1,2,:) + C(2,1,:) + C(2,2,:), [3 1 2]);
TP = permute(C(2,2,:), [3 1 2]);
TN = permute(C(1,1,:), [3 1 2]);
FP = permute(C(1,2,:), [3 1 2]);
FN = permute(C(2,1,:), [3 1 2]);

% to avoid division by zero
TPFP = TP+FP;
TPFN = TP+FN;
FPTN = FP+TN;
I_TPFP = find(TPFP > 0);
I_TPFN = find(TPFN > 0);
I_FPTN = find(FPTN > 0);

Stats.Prevalence = (TP+FN)./all;
Stats.Accuracy = (TP+TN)./all;
Stats.PositiveTendency = TPFP./all;
Stats.Discovery = TP./all;

Stats.Precision = ones(size(TP));
Stats.Precision(I_TPFP) = TP(I_TPFP) ./ TPFP(I_TPFP);

Stats.Recall = ones(size(TP));
Stats.Recall(I_TPFN) = TP(I_TPFN) ./ TPFN(I_TPFN);

Stats.Specificity = ones(size(TN));
Stats.Specificity(I_FPTN) = TN(I_FPTN) ./ FPTN(I_FPTN);

Stats.FMeasure = zeros(size(TP));
PrecRec = Stats.Precision + Stats.Recall;
I_PrecRec = find(PrecRec > 0);
Stats.FMeasure(I_PrecRec) = 2 .* Stats.Precision(I_PrecRec) .* Stats.Recall(I_PrecRec) ./ PrecRec(I_PrecRec);

if nargin >= 2
    % include Adjusted Precision
    TPAFP = (TP + N1N2adjustment .* FP);
    Stats.AdjustedPrecision = ones(size(TP));
    Stats.AdjustedPrecision(I_TPFP) = TP(I_TPFP) ./ TPAFP(I_TPFP);
end

Stats.AUC = (Stats.Recall + Stats.Specificity) / 2;

return
