function [OldArea, NewArea] = GetROC_compare(Labels, OldOutputs, NewOutputs, Captions, draw, saveFileName, saveFormat)
% [OldArea, NewArea] = GetROC_compare(Labels, OldOutputs, NewOutputs, draw, saveFileName, saveFormat)
%
% <Labels> and <Outputs> are column vectors.
% <draw> = 'draw' plots the ROC curve.
%   default is empty ('') value, which does not plot (except if <filename> given)
% if <draw> = 'draw' and <saveFileName> specified, the figure is PRINTed to file.
% <saveFormat> is 'png' by default is passed as '-d<saveFormat>' to PRINT.
% 'draw', '<filename>' will plot, save and keep the figure open.
% '', '<filename>' will plot, save and close the figure.

if nargin < 4, Captions = {}; end;
if nargin < 5, draw = ''; end;
if nargin < 6, saveFileName = ''; end;
if nargin < 7, saveFormat = 'png'; end;

if isempty(Captions)
  Captions = {'Before', 'After'}
end

% write AUC changes
I_rows = find(~isnan(OldOutputs));
thisOldOut = OldOutputs(I_rows);
thisNewOut = NewOutputs(I_rows);
thisLabels = Labels(I_rows);

[OldArea, OldSens, OldNSpec] = GetROC(thisLabels, thisOldOut);
[NewArea, NewSens, NewNSpec] = GetROC(thisLabels, thisNewOut);

if isequal(draw, 'draw') | ~isempty(saveFileName)
    figure;
    hold on;
    title(['ROC curves (Area ',Captions{1},'=', num2str(OldArea) ,' ; ',Captions{2},'=',num2str(NewArea),' ; \Delta=', num2str(NewArea-OldArea) ,')']);
    ylabel('Sensitivity');
    xlabel('1 - Specificity');
    ylim([0 1.01]);
    HOld = plot(OldNSpec, OldSens, 'k:');
    set(HOld, 'LineWidth', 1);
    
    HNew = plot(NewNSpec, NewSens, 'k-');
    set(HNew, 'LineWidth', 1);

    legend(Captions{1}, Captions{2}, 'Location', 'SouthEast');

    if ~isempty(saveFileName)
        print(['-d', saveFormat], saveFileName);
    end
    
    if ~isequal(draw, 'draw')
        close;
    end
end
