function drawSynthHierarchyPlotMatrices(hierarchyNames, dataroot, direction, micromacro)
% Create the following plots:
%   microaveraged F-measures per prevalence * recall * specificity.
%   macroaveraged
%   microaveraged movement in the recall-specificity plane per prevalence
%   macroaveraged

%   * movement in the micro/macro P-R plane (2D)
%   *  3D breakdowns by depth
%      y: vertical arrows from old to new micro/macro F-measure
%   3. scatterplot of new vs. old micro/macro F-measure
%

savefigs = '';

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1, hierarchyNames = {}; end
if isempty(hierarchyNames)
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
    'NT5D3'
    'NT5D4'
    'RT4D5'
    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

if nargin < 3, direction = []; end
if isempty(direction)
  direction = 'down';
end

if nargin < 4, micromacro = 'macro'; end

%
% Create figures
%
%maxdepth = 6;
hAccF = figure;
%axis manual;
%axis([0 1 0 1]);
%title('Microaveraged F-measure');
%xlabel('1 - Specificity (FP rate)');
%ylabel('Sensitivity (TP rate)');
hold on;

subplot_rows = length(hierarchyNames);
subplot_cols = 5;
marx=0.05;
mary=0.05;
subw=((1-marx)/subplot_cols) - 0.01;
subh=((1-mary)/subplot_rows) - 0.01;
H_subplots = zeros(subplot_rows, subplot_cols);
for i=1:subplot_rows
  for j=1:subplot_cols
    x = marx + (j-1) * ((1-marx)/subplot_cols);
    y = mary + (subplot_rows-i) * ((1-mary)/subplot_rows);
    H_subplots(i,j) = subplot('Position', [x,y,subw,subh]);
    hold on;
    axis([0 1 0 1]);
    set(H_subplots(i,j), 'XTickLabelMode', 'manual');
    set(H_subplots(i,j), 'YTickLabelMode', 'manual');
    set(H_subplots(i,j), 'XTick', [0:0.2:1]);
    set(H_subplots(i,j), 'YTick', [0:0.2:1]);
    set(H_subplots(i,j), 'XTickLabel', {});
    set(H_subplots(i,j), 'YTickLabel', {});
  end
end


for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);
  classes = hier.NumClasses;

  i_prevalence = 0;
  for prevalence = [.01, .05, .10, .25, .50]
    i_prevalence = i_prevalence + 1;
    
    OldAcc = [];
    OldF = [];
    NewF = [];
    
%    hMicroF = figure;
%    axis manual;
%    axis([0 1 0 1 0 1]);
%    title(sprintf('Microaveraged F-measure (%2.0f%% positive on %s)', prevalence * 100, hierName));
%    xlabel('1 - Specificity (FP rate)');
%    ylabel('Sensitivity (TP rate)');
%    zlabel('F-measure');
%    hold on;
%    hMacroF = figure;
%    axis manual;
%    axis([0 1 0 1 0 1]);
%    title(sprintf('Macroaveraged F-measure (%2.0f%% positive on %s)', prevalence * 100, hierName));
%    xlabel('1 - Specificity (FP rate)');
%    ylabel('Sensitivity (TP rate)');
%    zlabel('F-measure');
%    hold on;

%    hMicroSR = figure;
%    axis manual;
%    axis([0 1 0 1]);
%    title(sprintf('Microaveraged ROC (%2.0f%% positive)', prevalence * 100));
%    xlabel('1 - Specificity (FP rate)');
%    ylabel('Sensitivity (TP rate)');
%    hold on;
%    hMacroSR = figure;
%    axis manual;
%    axis([0 1 0 1]);
%    xlabel('1 - Specificity (FP rate)');
%    ylabel('Sensitivity (TP rate)');
%    title(sprintf('Macroaveraged ROC (%2.0f%% positive)', prevalence * 100));
%    hold on;

%    hMacroPR = figure;
%    axis manual;
%    axis([0 1 0 1]);
%    xlabel('Recall');
%    ylabel('Precision');
%    title(sprintf('Macroaveraged P-R (%2.0f%% positive)', prevalence * 100));
%    hold on;
%    [X, Y] = meshgrid(0:0.01:1, 0:0.01:1);
%    Z = X + Y;
%    Z(Z == 0) = 1;
%    Z = 2 .* X .* Y ./ Z;
%    contour(X,Y,Z,'k:');
%    
%    hMicroPR = figure;
%    axis manual;
%    axis([0 1 0 1]);
%    xlabel('Recall');
%    ylabel('Precision');
%    title(sprintf('Microaveraged P-R (%2.0f%% positive)', prevalence * 100));
%    hold on;
%    contour(X,Y,Z,'k:');

    for specificity = [.10, .30, .50, .70, .90]
      for recall = [.15, .35, .55, .75, .95]
        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('p%02.0f_s%02.0f_r%02.0f', prevalence * 100, specificity * 100, recall * 100);
        %disp(sprintf('Generating dataset %s for %s...', datasetName, hierName));
        
        datasetDir = [hierDir, '/', datasetName];
            
        Labels = load([datasetDir, '/Labels.mat']);
        Labels = Labels.Labels;
        Outputs = load([datasetDir, '/Outputs.mat']);
        Outputs = Outputs.Outputs;
        Predictions = load([datasetDir, sprintf('/bnet_%s.pred.mat', direction)]);
        Predictions = Predictions.Predictions;
        
        MacroA0 = 0;
        MacroA1 = 0;
        MacroF0 = 0;
        MacroF1 = 0;
        MacroS0 = 0;
        MacroS1 = 0;
        MacroR0 = 0;
        MacroR1 = 0;
        MacroP0 = 0;
        MacroP1 = 0;
        MicroC0 = zeros(2, 2);
        MicroC1 = zeros(2, 2);
        for i=1:classes
          C0 = GetConfusion(Labels(:,i), Outputs(:,i));
          C1 = GetConfusion(Labels(:,i), Predictions(:,i));
          CS0 = GetConfusionStats(C0);
          CS1 = GetConfusionStats(C1);
          MicroC0 = MicroC0 + C0;
          MicroC1 = MicroC1 + C1;
          MacroF0 = MacroF0 + CS0.FMeasure;
          MacroF1 = MacroF1 + CS1.FMeasure;
          MacroS0 = MacroS0 + CS0.Specificity;
          MacroS1 = MacroS1 + CS1.Specificity;
          MacroR0 = MacroR0 + CS0.Recall;
          MacroR1 = MacroR1 + CS1.Recall;
          MacroP0 = MacroP0 + CS0.Precision;
          MacroP1 = MacroP1 + CS1.Precision;
          MacroA0 = MacroA0 + CS0.Accuracy;
          MacroA1 = MacroA1 + CS1.Accuracy;
        end
        MacroF0 = MacroF0 ./ classes;
        MacroF1 = MacroF1 ./ classes;
        MacroS0 = MacroS0 ./ classes;
        MacroS1 = MacroS1 ./ classes;
        MacroR0 = MacroR0 ./ classes;
        MacroR1 = MacroR1 ./ classes;
        MacroP0 = MacroP0 ./ classes;
        MacroP1 = MacroP1 ./ classes;
        MacroA0 = MacroA0 ./ classes;
        MacroA1 = MacroA1 ./ classes;
        MicroCS0 = GetConfusionStats(MicroC0);
        MicroCS1 = GetConfusionStats(MicroC1);
        MicroF0 = MicroCS0.FMeasure;
        MicroF1 = MicroCS1.FMeasure;
        MicroS0 = MicroCS0.Specificity;
        MicroS1 = MicroCS1.Specificity;
        MicroR0 = MicroCS0.Recall;
        MicroR1 = MicroCS1.Recall;
        MicroP0 = MicroCS0.Precision;
        MicroP1 = MicroCS1.Precision;
        
        MicroA0 = MicroCS0.Accuracy;
        MicroA1 = MicroCS1.Accuracy;
 
        OldAcc(end+1) = MicroA0;
        if isequal(micromacro, 'micro')
          OldF(end+1) = MicroF0;
          NewF(end+1) = MicroF1;
        else
          OldF(end+1) = MacroF0;
          NewF(end+1) = MacroF1;
        end
%        % plot 3D F micro & macro graphs
%        figure(hMicroF);
%        plotbubble(1 - MicroS0, MicroR0, MicroF1 - MicroF0);
%%        arrow([1 - MicroS0, MicroR0, MicroF0], [1 - MicroS1, MicroR1, MicroF1]);
%        figure(hMacroF);
%        plotbubble(1 - MacroS0, MacroR0, MacroF1 - MacroF0);
%        arrow([1 - MacroS0, MacroR0, MacroF0], [1 - MacroS1, MacroR1, MacroF1]);

%        figure(hMicroSR);
%        arrow([1 - MicroS0, MicroR0], [1 - MicroS1, MicroR1]);
%        figure(hMacroSR);
%        arrow([1 - MacroS0, MacroR0], [1 - MacroS1, MacroR1]);
%        figure(hMicroPR);
%        arrow([MicroR0, MicroP0], [MicroR1, MicroP1]);
%        figure(hMacroPR);
%        arrow([MacroR0, MacroP0], [MacroR1, MacroP1]);
      end
    end
    
%    % Save plots
%    if isequal(savefigs, 'save')
%      figure(hMicroF);
%      print(sprintf('%s/bubble_%s_microF_p%02.0f.eps', dataroot, hierName, prevalence * 100), '-depsc2');
%      figure(hMacroF);
%      print(sprintf('%s/bubble_%s_macroF_p%02.0f.eps', dataroot, hierName, prevalence * 100), '-depsc2');
%    end

    figure(hAccF);
    subplot(H_subplots(iHier, i_prevalence));
    plotAccuracyF(OldAcc, OldF, NewF);

    if i_prevalence == 1
      figure(hAccF);
      ylabel(hierName);
    end
    if iHier==length(hierarchyNames)
      figure(hAccF);
      xlabel(sprintf('%2.0d%% Positive', prevalence * 100));
    end
  end
end

function plotbubble(X, Y, Z)
if Z > 0, mfc = 'black'; else mfc = 'white'; Z = -Z; end
plot(X, Y, 'ko', 'MarkerSize', Z * 47 + 3, 'MarkerFaceColor', mfc);

function plotAccuracyF(Acc, OldF, NewF)
%plot(Acc, OldF, 'k+');
%plot(Acc, NewF, 'k.');
for i=1:length(Acc)
  if OldF(i) < NewF(i)
    plot([Acc(i) Acc(i)], [OldF(i) NewF(i)], 'k-')
    plot(Acc(i), NewF(i), 'k^')
  else
    plot([Acc(i) Acc(i)], [OldF(i) NewF(i)], 'k-')
    plot(Acc(i), NewF(i), 'kv')
  end
end
