function drawSynthHierarchyPlotMatrices(hierarchyNames, dataroot, direction, micromacro, plotchoice, compare_to_bnet_none)
% Create the following plots:
%   microaveraged F-measures per prevalence * recall * specificity.
%   macroaveraged
%   microaveraged movement in the recall-specificity plane per prevalence
%   macroaveraged

%   * movement in the micro/macro P-R plane (2D)
%   *  3D breakdowns by depth
%      y: vertical arrows from old to new micro/macro F-measure
%   3. scatterplot of new vs. old micro/macro F-measure
%

savefigs = '';

if nargin < 6, compare_to_bnet_none = 0; end

if nargin < 5, plotchoice = 'scatter'; end
  xaxisvar = 'F';
  plottype = 'scatter';
if isequal(plotchoice, 'scatter')
elseif isequal(plotchoice, 'accuracy')
  xaxisvar = 'accuracy';
  plottype = 'arrows';
elseif isequal(plotchoice, 'precision')
  xaxisvar = 'precision';
  plottype = 'arrows';
else
  error('Invalid plot choice');
end

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1, hierarchyNames = {}; end
if isempty(hierarchyNames)
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
    'NT5D3'
    'NT5D4'
    'RT4D5'
    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

if nargin < 3, direction = []; end
if isempty(direction)
  direction = 'down';
end

if nargin < 4, micromacro = 'macro'; end

%
% Create figures
%
%maxdepth = 6;
hAccF = figure;
%axis manual;
%axis([0 1 0 1]);
%title('Microaveraged F-measure');
%xlabel('1 - Specificity (FP rate)');
%ylabel('Sensitivity (TP rate)');
hold on;

subplot_rows = length(hierarchyNames);
subplot_cols = 5;
marx=0.05;
mary=0.05;
subw=((1-marx)/subplot_cols) - 0.01;
subh=((1-mary)/subplot_rows) - 0.01;
H_subplots = zeros(subplot_rows, subplot_cols);
for i=1:subplot_rows
  for j=1:subplot_cols
    x = marx + (j-1) * ((1-marx)/subplot_cols);
    y = mary + (subplot_rows-i) * ((1-mary)/subplot_rows);
    H_subplots(i,j) = subplot('Position', [x,y,subw,subh]);
    hold on;
    axis([0 1 0 1]);
    set(H_subplots(i,j), 'XTickLabelMode', 'manual');
    set(H_subplots(i,j), 'YTickLabelMode', 'manual');
    set(H_subplots(i,j), 'XTick', [0:0.2:1]);
    set(H_subplots(i,j), 'YTick', [0:0.2:1]);
    set(H_subplots(i,j), 'XTickLabel', {});
    set(H_subplots(i,j), 'YTickLabel', {});
  end
end


for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);
  classes = hier.NumClasses;

  i_prevalence = 0;
  for prevalence = [.01, .05, .10, .25, .50]
    i_prevalence = i_prevalence + 1;
    
    OldAcc = [];
    OldF = [];
    NewF = [];

    for specificity = [.10, .30, .50, .70, .90]
      for recall = [.15, .35, .55, .75, .95]
        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('p%02.0f_s%02.0f_r%02.0f', prevalence * 100, specificity * 100, recall * 100);
        %disp(sprintf('Processing dataset %s for %s...', datasetName, hierName));
        
        datasetDir = [hierDir, '/', datasetName];

        if compare_to_bnet_none
          Conf0 = load([datasetDir, '/Confusions_bnet_none.mat']);
        else
          Conf0 = load([datasetDir, '/Confusions.mat']);
        end
        Conf1 = load([datasetDir, sprintf('/Confusions_bnet_%s.mat', direction)]);
        
        MacroA0 = 0;
        MacroA1 = 0;
        MacroF0 = 0;
        MacroF1 = 0;
        MacroS0 = 0;
        MacroS1 = 0;
        MacroR0 = 0;
        MacroR1 = 0;
        MacroP0 = 0;
        MacroP1 = 0;
        for i=1:classes
          CS0 = GetConfusionStats(Conf0.C{i});
          CS1 = GetConfusionStats(Conf1.C{i});
          MacroF0 = MacroF0 + CS0.FMeasure;
          MacroF1 = MacroF1 + CS1.FMeasure;
          MacroS0 = MacroS0 + CS0.Specificity;
          MacroS1 = MacroS1 + CS1.Specificity;
          MacroR0 = MacroR0 + CS0.Recall;
          MacroR1 = MacroR1 + CS1.Recall;
          MacroP0 = MacroP0 + CS0.Precision;
          MacroP1 = MacroP1 + CS1.Precision;
          MacroA0 = MacroA0 + CS0.Accuracy;
          MacroA1 = MacroA1 + CS1.Accuracy;
        end
        MacroF0 = MacroF0 ./ classes;
        MacroF1 = MacroF1 ./ classes;
        MacroS0 = MacroS0 ./ classes;
        MacroS1 = MacroS1 ./ classes;
        MacroR0 = MacroR0 ./ classes;
        MacroR1 = MacroR1 ./ classes;
        MacroP0 = MacroP0 ./ classes;
        MacroP1 = MacroP1 ./ classes;
        MacroA0 = MacroA0 ./ classes;
        MacroA1 = MacroA1 ./ classes;
        MicroCS0 = GetConfusionStats(Conf0.TotalC);
        MicroCS1 = GetConfusionStats(Conf1.TotalC);
        MicroF0 = MicroCS0.FMeasure;
        MicroF1 = MicroCS1.FMeasure;
        MicroS0 = MicroCS0.Specificity;
        MicroS1 = MicroCS1.Specificity;
        MicroR0 = MicroCS0.Recall;
        MicroR1 = MicroCS1.Recall;
        MicroP0 = MicroCS0.Precision;
        MicroP1 = MicroCS1.Precision;
        
        MicroA0 = MicroCS0.Accuracy;
        MicroA1 = MicroCS1.Accuracy;
 
        if isequal(xaxisvar, 'accuracy')
          OldAcc(end+1) = MacroA0;
        elseif isequal(xaxisvar, 'precision')
          OldAcc(end+1) = MacroP0;
        elseif isequal(xaxisvar, 'F')
          % do nothing, but don't give error.
        else
          error('bug: invalid xaxisvar');
        end
        if isequal(micromacro, 'micro')
          OldF(end+1) = MicroF0;
          NewF(end+1) = MicroF1;
        elseif isequal(micromacro, 'macro')
          OldF(end+1) = MacroF0;
          NewF(end+1) = MacroF1;
        else
          error('Invalid micro/macro parameter');
        end
      end
    end
    
    subplot(H_subplots(iHier, i_prevalence));
    if isequal(plottype, 'arrows')
      plotArrows(OldAcc, OldF, NewF);
    elseif isequal(plottype, 'scatter')
      plot(OldF, NewF, 'k.');
      plot([0 1], [0 1], 'k--');
    else
      error('bug: invalid xaxisvar');
    end

    if i_prevalence == 1
      ylabel(hierName);
    end
    if iHier==length(hierarchyNames)
      xlabel(sprintf('%2.0d%% Positive', prevalence * 100));
    end
  end
end

function plotbubble(X, Y, Z)
if Z > 0, mfc = 'black'; else mfc = 'white'; Z = -Z; end
plot(X, Y, 'ko', 'MarkerSize', Z * 47 + 3, 'MarkerFaceColor', mfc);

function plotArrows(Acc, OldF, NewF)
%plot(Acc, OldF, 'k+');
%plot(Acc, NewF, 'k.');
for i=1:length(Acc)
  if OldF(i) < NewF(i)
    plot([Acc(i) Acc(i)], [OldF(i) NewF(i)], 'k-')
    plot(Acc(i), NewF(i), 'k^')
  else
    plot([Acc(i) Acc(i)], [OldF(i) NewF(i)], 'k-')
    plot(Acc(i), NewF(i), 'kv')
  end
end
