function summarizeSynthHierarchyResults_topdown2(hierarchyNames, dataroot, direction)
% function summarizeSynthHierarchyResults(hierarchyNames, dataroot, direction)

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1, hierarchyNames = {}; end
if isempty(hierarchyNames)
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
%    'NT5D3'
%    'NT5D4'
%    'RT4D5'
%    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

if nargin < 3, direction = []; end
if isempty(direction)
  direction = 'down';
end

for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);
  classes = hier.NumClasses;
  [mindepths, maxdepths, numchildren] = getNodeStats(hier.NodeParents, hier.NumClasses);

  for prevalence = [.01] % [.01, .05, .10, .25, .50]
    for accuracy = [.60, .70, .80, .90]
        specificity = accuracy;
        recall = accuracy;
        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('p%02.0f_a%02.0f', prevalence * 100, accuracy * 100);
        %disp(sprintf('Processing dataset %s for %s...', datasetName, hierName));
        
        datasetDir = [hierDir, '/', datasetName];

        %if compare_to_bnet_none
        %  Conf0 = load([datasetDir, '/Confusions_bnet_none.mat']);
        %else
          Conf0 = load([datasetDir, '/Confusions.mat']);
        %end
        Conf1 = load([datasetDir, sprintf('/Confusions_bnet_%s.mat', direction)]);
        
        MacroF0 = 0;
        MacroF1 = 0;
        MacroS0 = 0;
        MacroS1 = 0;
        MacroR0 = 0;
        MacroR1 = 0;
        MacroP0 = 0;
        MacroP1 = 0;
        for i=1:classes
          CS0 = GetConfusionStats(Conf0.C{i});
          CS1 = GetConfusionStats(Conf1.C{i});
          MacroF0 = MacroF0 + CS0.FMeasure;
          MacroF1 = MacroF1 + CS1.FMeasure;
          MacroS0 = MacroS0 + CS0.Specificity;
          MacroS1 = MacroS1 + CS1.Specificity;
          MacroR0 = MacroR0 + CS0.Recall;
          MacroR1 = MacroR1 + CS1.Recall;
          MacroP0 = MacroP0 + CS0.Precision;
          MacroP1 = MacroP1 + CS1.Precision;
        end
        MacroF0 = MacroF0 ./ classes;
        MacroF1 = MacroF1 ./ classes;
        MacroS0 = MacroS0 ./ classes;
        MacroS1 = MacroS1 ./ classes;
        MacroR0 = MacroR0 ./ classes;
        MacroR1 = MacroR1 ./ classes;
        MacroP0 = MacroP0 ./ classes;
        MacroP1 = MacroP1 ./ classes;
        MicroCS0 = GetConfusionStats(Conf0.TotalC);
        MicroCS1 = GetConfusionStats(Conf1.TotalC);
        MicroF0 = MicroCS0.FMeasure;
        MicroF1 = MicroCS1.FMeasure;
        MicroS0 = MicroCS0.Specificity;
        MicroS1 = MicroCS1.Specificity;
        MicroR0 = MicroCS0.Recall;
        MicroR1 = MicroCS1.Recall;
        MicroP0 = MicroCS0.Precision;
        MicroP1 = MicroCS1.Precision;
        
        A0 = MicroCS0.Accuracy;
        A1 = MicroCS1.Accuracy;
        
        s = [hierName, '/', datasetName, sprintf(' & %5.1f', [A0 A1 A1-A0] * 100)];

        for i=0:max(mindepths)
          I_rowclasses = find(mindepths == i);
          MicroC0 = zeros(2, 2);
          MicroC1 = zeros(2, 2);
          for j=I_rowclasses
            MicroC0 = MicroC0 + Conf0.C{j};
            MicroC1 = MicroC1 + Conf1.C{j};
          end
          MicroCS0 = GetConfusionStats(MicroC0);
          MicroCS1 = GetConfusionStats(MicroC1);
          A0 = MicroCS0.Accuracy;
          A1 = MicroCS1.Accuracy;
          
          if MicroC1(1,1) == 0 || MicroC1(2,2) == 0,
            s = [s, sprintf(' & %5.1f*', [A1-A0] * 100)];
          else
            s = [s, sprintf(' & %5.1f', [A1-A0] * 100)];
          end
        end

        disp(s);
        
      end
%    end
  end
end
