function summarizeSynthHierarchyResults_topdown(hierarchyNames, dataroot, direction)
% function summarizeSynthHierarchyResults(hierarchyNames, dataroot, direction)

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1, hierarchyNames = {}; end
if isempty(hierarchyNames)
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
%    'NT5D3'
%    'NT5D4'
%    'RT4D5'
%    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

if nargin < 3, direction = []; end
if isempty(direction)
  direction = 'down';
end

for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);
  classes = hier.NumClasses;

  for prevalence = .50
    for specificity = .99
      for recall = .35:.10:.95
        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('q%02.0f_s%02.0f_r%02.0f', prevalence * 100, specificity * 100, recall * 100);
        %disp(sprintf('Processing dataset %s for %s...', datasetName, hierName));
        
        datasetDir = [hierDir, '/', datasetName];

        %if compare_to_bnet_none
        %  Conf0 = load([datasetDir, '/Confusions_bnet_none.mat']);
        %else
          Conf0 = load([datasetDir, '/Confusions.mat']);
        %end
        Conf1 = load([datasetDir, sprintf('/Confusions_bnet_%s.mat', direction)]);
        
        MacroA0 = 0;
        MacroA1 = 0;
        MacroF0 = 0;
        MacroF1 = 0;
        MacroS0 = 0;
        MacroS1 = 0;
        MacroR0 = 0;
        MacroR1 = 0;
        MacroP0 = 0;
        MacroP1 = 0;
        for i=1:classes
          CS0 = GetConfusionStats(Conf0.C{i});
          CS1 = GetConfusionStats(Conf1.C{i});
          MacroF0 = MacroF0 + CS0.FMeasure;
          MacroF1 = MacroF1 + CS1.FMeasure;
          MacroS0 = MacroS0 + CS0.Specificity;
          MacroS1 = MacroS1 + CS1.Specificity;
          MacroR0 = MacroR0 + CS0.Recall;
          MacroR1 = MacroR1 + CS1.Recall;
          MacroP0 = MacroP0 + CS0.Precision;
          MacroP1 = MacroP1 + CS1.Precision;
          MacroA0 = MacroA0 + CS0.Accuracy;
          MacroA1 = MacroA1 + CS1.Accuracy;
        end
        MacroF0 = MacroF0 ./ classes;
        MacroF1 = MacroF1 ./ classes;
        MacroS0 = MacroS0 ./ classes;
        MacroS1 = MacroS1 ./ classes;
        MacroR0 = MacroR0 ./ classes;
        MacroR1 = MacroR1 ./ classes;
        MacroP0 = MacroP0 ./ classes;
        MacroP1 = MacroP1 ./ classes;
        MacroA0 = MacroA0 ./ classes;
        MacroA1 = MacroA1 ./ classes;
        MicroCS0 = GetConfusionStats(Conf0.TotalC);
        MicroCS1 = GetConfusionStats(Conf1.TotalC);
        MicroF0 = MicroCS0.FMeasure;
        MicroF1 = MicroCS1.FMeasure;
        MicroS0 = MicroCS0.Specificity;
        MicroS1 = MicroCS1.Specificity;
        MicroR0 = MicroCS0.Recall;
        MicroR1 = MicroCS1.Recall;
        MicroP0 = MicroCS0.Precision;
        MicroP1 = MicroCS1.Precision;
        
        MicroA0 = MicroCS0.Accuracy;
        MicroA1 = MicroCS1.Accuracy;
         
        disp([hierName, '/', datasetName, sprintf('\t%g', [
          MicroA1-MicroA0
          MacroA1-MacroA0
          MicroF1-MicroF0
          MacroF1-MacroF0
        ])]);
        
      end
    end
  end
end
