function chunkData(chunkSize, InputFileNames, OutputPattern, varName)
% function chunkData(chunkSize, InputFileNames, OutputPattern, varName)

if nargin < 4
  varName = 'ALL_inputs';
end

if nargin < 3
  OutputPattern = 'chunks/lyrl2004_vectors_test.%d.chunk.mat';
end

if nargin < 2
  InputFileNames = {
    'lyrl2004_vectors_test_pt0.mat'
    'lyrl2004_vectors_test_pt1.mat'
    'lyrl2004_vectors_test_pt2.mat'
    'lyrl2004_vectors_test_pt3.mat'
  };
end


% Save chunks.
chunkCount = 0;
Overflow = zeros(0,0);
for curFile = 1:length(InputFileNames)
  disp(['Loading ', InputFileNames{curFile}, '...']);
  Inputs = loadInputsFile(InputFileNames{curFile}, varName);
  if size(Overflow, 1) > 0
    Inputs = [Overflow ; Inputs];
  end
  
  numChunks = div(size(Inputs, 1), chunkSize);
  disp(['Writing ', num2str(numChunks), ' chunks...']);
  for curChunk = 1:numChunks
    iFirst = (curChunk - 1) * chunkSize + 1;
    iLast = curChunk * chunkSize;
    Chunk = Inputs(iFirst:iLast, :);
    chunkCount = chunkCount + 1;
    
    eval([varName ' = Chunk;']);
    save(sprintf(OutputPattern, chunkCount), varName);
  end
  
  Overflow = Inputs(iLast+1:end, :);
end

% Save overflow chunk, if any.
if size(Overflow, 1) > 0
  Chunk = Overflow;
  chunkCount = chunkCount + 1;
  
  eval([varName ' = Chunk;']);
  save(sprintf(OutputPattern, chunkCount), varName);
end

disp(sprintf('%d chunks saved.', chunkCount));

return
%-----------------------------------------------------------------------------
function Inputs = loadInputsFile(fname, varName)
if (isequal(fname(end-3:end), '.mat'))
    Inputs = load(fname);
    eval(['Inputs = Inputs.', varName,';']);
else
    Inputs = spconvert(dataread('file', fname, '', -1,'delimiter','\t'));
end
return
