function Fields = joinChunks(numChunks, ChunksPattern, JoinedFile)
% function Fields = joinChunks(numChunks, ChunksPattern, [JoinedFile])
%
% Can join any deep struct.

% Load each chunk.
disp(['Loading chunk ', num2str(1), '...']);
Fields = load(sprintf(ChunksPattern, 1));
for i = 2:numChunks
  disp(['Joining chunk ', num2str(i), '...']);
  NewFields = load(sprintf(ChunksPattern, i));
  Fields = joinVars(Fields, NewFields);
  clear NewFields;
  pack;
end

disp('Saving joint file...');
% Save if filename specified.
if ~isempty(JoinedFile)
  save(JoinedFile, '-struct', Fields);
end

disp(['Done! ', num2str(numChunks), ' chunks joined.']);
return

%------------------------------------------------------------------
function X = joinVars(L, R)
if isstruct(L)
  X = struct;
  lnames = fieldnames(L);
  rnames = fieldnames(R);
  assert(isequal(lnames, rnames), 'Field names mismatch');
  for i=1:length(lnames)
    X.(lnames{i}) = joinVars(L.(lnames{i}), R.(rnames{i}));
  end
else
  X = [L ; R];
end
return
