function Fields = joinPredictionChunks(numChunks, ChunksPattern, JoinedFile, totalRows, numClasses)
% function Fields = joinPredictionChunks(numChunks, ChunksPattern, JoinedFile, totalRows, numClasses)
%
% Data field names are hard-coded.

numAddedRows = 0;
for i = 1:numChunks
  disp(['Joining chunk ', num2str(i), '...']);
  NewPredictions = load(sprintf(ChunksPattern, i));
  NewPredictions = NewPredictions.Predictions;
  
  if isstruct(NewPredictions)
    if i == 1
      Predictions = struct;
    end

  
    fields = fieldnames(NewPredictions);
    for j = 1:length(fields)
      fld = fields{j};
      
      % Create Predictions field if first file.
      if i == 1
        Predictions.(fld) = zeros(totalRows, numClasses);
      end
      
      % Get number of rows if first field.
      if j == 1
        numNewRows = size(NewPredictions.(fld), 1);
      else
        assert( numNewRows == size(NewPredictions.(fld), 1) );
      end
      Predictions.(fld)((numAddedRows+1):(numAddedRows+numNewRows), :) = NewPredictions.(fld);
    end
  
  else % not struct
    if i == 1
      Predictions = zeros(totalRows, numClasses);
    end
    
    % Get number of rows if first field.
    numNewRows = size(NewPredictions, 1);
    Predictions((numAddedRows+1):(numAddedRows+numNewRows), :) = NewPredictions;
  end
  
  numAddedRows = numAddedRows + numNewRows;
end

if numAddedRows ~= totalRows
  disp(sprintf('WARNING: %d rows expected, %d rows joined!', totalRows, numAddedRows));
end

disp('Saving joint file...');
% Save if filename specified.
if ~isempty(JoinedFile)
  save(JoinedFile, 'Predictions');
end

disp(['Done! ', num2str(numChunks), ' chunks joined.']);
return
