function joinSynthHierarchyPredictionChunks(hierarchyNames, dataroot, rows)

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
%    'NT5D3'
%    'NT5D4'
%    'RT4D5'
%    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

if nargin < 3
  rows = 10000;
end

for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);

  for prevalence = [.50] % [.01, .05, .10, .25, .50]
    for specificity = [.99] % [.10, .30, .50, .70, .90]
      for recall = .35 : .10 : .95 % [.15, .35, .55, .75, .95]
        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('q%02.0f_s%02.0f_r%02.0f', prevalence * 100, specificity * 100, recall * 100);
        
        datasetDir = [hierDir, '/', datasetName];

        
        chunkPattern = [datasetDir, '/chunks/bnet_down.%03d.pred.mat']; 
        jointFile = [datasetDir, '/bnet_down.pred.mat'];
        if ~exist(jointFile, 'file') && exist(sprintf(chunkPattern, 1), 'file')
          disp(['Joining bnet_down for ', hierName, '/', datasetName, '...']);
          joinPredictionChunks(10, chunkPattern, jointFile, rows, hier.NumClasses);
        end

        chunkPattern = [datasetDir, '/chunks/bnet_none.%03d.pred.mat']; 
        jointFile = [datasetDir, '/bnet_none.pred.mat'];
        if ~exist(jointFile, 'file') && exist(sprintf(chunkPattern, 1), 'file')
          disp(['Joining bnet_none for ', hierName, '/', datasetName, '...']);
          joinPredictionChunks(10, chunkPattern, jointFile, rows, hier.NumClasses);
        end

        chunkPattern = [datasetDir, '/chunks/bnet_up.%03d.pred.mat']; 
        jointFile = [datasetDir, '/bnet_up.pred.mat'];
        if ~exist(jointFile, 'file') && exist(sprintf(chunkPattern, 1), 'file')
          disp(['Joining bnet_up for ', hierName, '/', datasetName, '...']);
          joinPredictionChunks(10, chunkPattern, jointFile, rows, hier.NumClasses);
        end

      end
    end
  end
end

disp('Done!')
