function [OldAreas, NewAreas] = loadAllAUCs(pattern, ClassIDs)
% function [Outputs] = loadAllAUCs(pattern, ClassIDs)
%
% Loads AUCs from submit_roc_selected's .auc.mat output files.
%
% <pattern> is filename pattern with %s for ClassIDs number.
%   default if empty: '~/research/function/data/outputs/outputs_%s.mat.txt'
% <numExamples> is the number of training examples.
% <ClassIDs> is array of ClassIDs to load labels for
%   default if empty: ClassIDs = getSelectedClassIDs;
%
% <Outputs> is held-out outputs (NumTrainingExamples-by-NumClassIDs)
%  If an example was not I_rows or held-out in all folds

if nargin < 2, ClassIDs = []; end;

if isempty(pattern)
    error('You must specify AUC file pattern');
end

if isempty(ClassIDs)
    ClassIDs = getSelectedClassIDs;
end


OldAreas = zeros(length(ClassIDs), 1);
NewAreas = zeros(length(ClassIDs), 1);

for i = 1:length(ClassIDs)
    classid = ClassIDs{i};
    disp(sprintf('Loading %s outputs (%d of %d)', classid, i, length(ClassIDs)));
    resultsfile = sprintf(pattern, classid);
    R = load(resultsfile);
    OldAreas(i, 1) = R.OldArea;
    NewAreas(i, 1) = R.NewArea;
end
