function Labels = loadLabels(pattern, ClassIDs, replaceNaN)
% function Labels = loadLabels(pattern, ClassIDs, replaceNaN)
%
% <pattern> is filename pattern with %s for ClassID number.
%   default if empty: '~/research/function/data/outputs/outputs_%s.mat.txt'
% <ClassIDs> is array of ClassIDs to load labels for
%   default if empty: ClassIDs = getSelectedClassIDs;
% <replaceNaN> is number to replace NaN labels with.
%   default if empty: (no replacing, leave as NaN)
%
% <Labels> is training labels (NumTrainingExamples-by-NumClassIDs)

if nargin < 3, replaceNaN = []; end;
if nargin < 2, ClassIDs = []; end;
if nargin < 1, pattern = ''; end

if isempty(pattern)
    %pattern = [getenv('HOME'), '/research/function/data/outputs/outputs_%s.mat.txt'];
    pattern = ['C:/Documents and Settings/Administrator/My Documents/Machine Learning/Shape/Data/outputs/outputs_%s.mat.txt'];
end

if isempty(ClassIDs)
    ClassIDs = getSelectedClassIDs;
end

% classid = ClassIDs{1};
% if ~exist(sprintf(pattern, classid), 'file'), error('Default labels path not found!'); end

Labels = [];
for i = 1:length(ClassIDs)
    classid = ClassIDs{i};
    labelsfile = sprintf(pattern, classid);
    Labels(:,i) = full(spconvert(dataread('file', labelsfile, '', -1,'delimiter','\t')));
end

if ~isempty(replaceNaN)
    Labels(find(isnan(Labels))) = replaceNaN;
end

