function NodeParents = loadNodeParents(filename, ClassIDList)
% function NodeParents = loadNodeParents(filename, ClassIDList)
% 
% Loads parentClassID -> ClassID pairs from file exported from Python by ShapeHierarchy.exportNodeParents.
% 
% if ClassIDList is specified, list indices of ClassIDs are returned instead.
% filename default if blank: '~/research/shape/data/nodeParents.mat.txt'

if nargin < 1, filename = ''; end;

if isempty(filename) 
    filename = [getenv('HOME'), '/research/function/Reuters/data/nodeParents.mat.txt'];
    %filename = ['C:/Documents and Settings/Administrator/My Documents/Machine Learning/Shape/Data/nodeParents.mat.txt'];
end


[ParentClassIDs, NodeClassIDs] = textread(filename, '%q%q');

if nargin > 1
    N = length(ParentClassIDs);
    K = length(ClassIDList);
    NodeParents = zeros(0, 2);
    for i = 1:N
        pidindex = 0;
        for j = 1:K
            if isequal(ParentClassIDs{i}, ClassIDList{j})
                pidindex = j;
                break;
            end
        end
        if pidindex == 0, continue; end;
    
        classidindex = 0;
        for j = 1:K
            if isequal(NodeClassIDs{i}, ClassIDList{j})
                classidindex = j;
                break;
            end
        end
        if classidindex == 0, continue; end;

        NodeParents(end+1,:) = [pidindex, classidindex];
    end
else
    NodeParents = [ParentClassIDs, NodeClassIDs];
end
