function [Outputs] = loadOutputsFromResults(pattern, numExamples, ClassIDs)
% function Outputs = loadOutputsFromResults(pattern, numExamples, ClassIDs)
%
% Loads held-out outputs of classifiers from Results.
%
% <pattern> is filename pattern with %s for ClassIDs number.
%   default if empty: '~/research/function/data/outputs/outputs_%s.mat.txt'
% <numExamples> is the number of training examples.
% <ClassIDs> is array of ClassIDs to load labels for
%   default if empty: ClassIDs = getSelectedClassIDs;
%
% <Outputs> is held-out outputs (NumTrainingExamples-by-NumClassIDs)
%  If an example was not I_rows or held-out in all folds

if nargin < 3, ClassIDs = []; end;

if nargin < 2
    error('You must specify numExamples');
end

if isempty(pattern)
    error('You must specify Result file pattern');
end

if isempty(ClassIDs)
    ClassIDs = getSelectedClassIDs;
end

Outputs = NaN(numExamples, length(ClassIDs));
K = zeros(length(ClassIDs), 1);
for i = 1:length(ClassIDs)
    classid = ClassIDs{i};
    disp(sprintf('Loading %s outputs (%d of %d)', classid, i, length(ClassIDs)));
    resultsfile = sprintf(pattern, classid);
    R = load(resultsfile);
%    if size(R.Results.I_rows,1) > 0
      Outputs(R.Results.I_rows,i) = R.Results.HeldOutMedianOutputs;
%    else
%      Outputs(:,i) = R.Results.HeldOutMedianOutputs;
%    end
end
