function [I_train, I_valid] = sampleByClass(Labels, ratio);
% function [I_train, I_valid] = sampleByClass(Labels, ratio);
% 
% Random-without-replacement sample by class, getting ratio
% of training examples from positives and negatives.
% Default for ratio is 0.90.
% I_train, I_valid are col vectors.

if nargin < 2, ratio = 0.90; end;

N = size(Labels, 1);

% use class sampling, since positives are rare
I_pos = find(Labels);
I_neg = find(Labels == 0);
N_pos = size(I_pos,1);
N_neg = size(I_neg,1);
if (N_pos + N_neg ~= N), error('BUG: Negative and positive counts do not add up'); end;
N_train_pos = floor(N_pos * ratio);
N_train_neg = floor(N_neg * ratio);
if (N_train_pos == 0) N_train_pos = 1; end;
if (N_train_neg == 0) N_train_neg = 1; end;
if (N_train_pos == N_pos) N_train_pos = N_pos-1; end;
if (N_train_neg == N_neg) N_train_neg = N_neg-1; end;
I_pos = I_pos(randperm(N_pos));
I_neg = I_neg(randperm(N_neg));
I_train = [I_pos(1:N_train_pos); I_neg(1:N_train_neg)];
I_valid = [I_pos(N_train_pos+1:end); I_neg(N_train_neg+1:end)];
I_train = I_train(randperm(size(I_train,1)));
I_valid = I_valid(randperm(size(I_valid,1)));
if (size(I_train,1) + size(I_valid,1) ~= N), error('BUG: N_train+N_valid ~= N'); end;
