function [Folds] = sampleFoldsByClass(Indices, Labels, k);
% function [IFolds] = sampleFoldsByClass(Indices, Labels, k);
% 
% Random-without-replacement sampling by class into k disjoint folds
% of training examples from positives and negatives.
%
% Folds is a cell array of structs.
% Folds{i}.IEval and Folds{i}.ITrain are col vectors containing indices.

N = size(Labels, 1);

% use class sampling, since positives are rare
I_pos = Indices(find(Labels));
I_neg = Indices(find(Labels == 0));
N_pos = size(I_pos,1);
N_neg = size(I_neg,1);
if (N_pos + N_neg ~= N), error('BUG: Negative and positive counts do not add up'); end;

Folds_pos = sampleFolds(I_pos, k);
Folds_neg = sampleFolds(I_neg, k);
Folds = cell(1, k);
for i=1:k,
    Folds{i} = struct;
    Folds{i}.ITrain = [Folds_pos{i}.ITrain ; Folds_neg{i}.ITrain];
    Folds{i}.IEval = [Folds_pos{i}.IEval ; Folds_neg{i}.IEval];
    % reshuffle so negatives aren't always at end
    Folds{i}.ITrain = Folds{i}.ITrain(randperm(size(Folds{i}.ITrain, 1)));
    Folds{i}.IEval = Folds{i}.IEval(randperm(size(Folds{i}.IEval, 1)));
end
