function saveSynthHierarchyConfusions(hierarchyNames, dataroot)
% Create the following plots:
%   microaveraged F-measures per prevalence * recall * specificity.
%   macroaveraged
%   microaveraged movement in the recall-specificity plane per prevalence
%   macroaveraged

%   * movement in the micro/macro P-R plane (2D)
%   *  3D breakdowns by depth
%      y: vertical arrows from old to new micro/macro F-measure
%   3. scatterplot of new vs. old micro/macro F-measure
%

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1, hierarchyNames = {}; end
if isempty(hierarchyNames)
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
%    'NT5D3'
%    'NT5D4'
%    'RT4D5'
%    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);
  classes = hier.NumClasses;

  for prevalence = [.75] % [.01, .05, .10, .25, .50]
    for accuracy = [.70, .80, .90]
        specificity = accuracy;
        recall = accuracy;
        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('q%02.0f_a%02.0f', prevalence * 100, accuracy * 100);
        %disp(sprintf('Generating dataset %s for %s...', datasetName, hierName));
        
        datasetDir = [hierDir, '/', datasetName];
            
        Labels = load([datasetDir, '/Labels.mat']);
        Labels = Labels.Labels;

        SaveConfusions(Labels, [datasetDir, '/Outputs.mat'], [datasetDir, '/Confusions.mat']);
        SaveConfusions(Labels, [datasetDir, '/bnet_down.pred.mat'], [datasetDir, '/Confusions_bnet_down.mat']);
        SaveConfusions(Labels, [datasetDir, '/bnet_up.pred.mat'], [datasetDir, '/Confusions_bnet_up.mat']);
        SaveConfusions(Labels, [datasetDir, '/bnet_none.pred.mat'], [datasetDir, '/Confusions_bnet_none.mat']);
      end
  %  end
    
  end
end

function SaveConfusions(Labels, predsfilename, savefilename);
Outputs = load(predsfilename);
if isfield(Outputs, 'Outputs')
  Outputs = Outputs.Outputs;
elseif isfield(Outputs, 'Predictions')
  Outputs = Outputs.Predictions;
else
  error('Field not found');
end
TotalC = zeros(2, 2);
for i=1:size(Labels, 2)
  C{i} = GetConfusion(Labels(:,i), Outputs(:,i));
  TotalC = TotalC + C{i};
end
save(savefilename, 'C', 'TotalC');
