function Results = exp_SVMlight(aggregation, aggrParams, classid, inputsfile, labelspath, resultsdir, kerneltype)
% function Results = exp_SVMlight(aggregation, aggrParams, classid, inputsfile, labelspath, resultsdir, kerneltype)
%
% <labelspath> can be one of:
%   <path_template>
%   <preloaded_matrix>
%   {<path_template>, <colindex>}
%   {<preloaded_matrix>, <colindex>}

% validate args
if ((nargin < 1) | isempty(aggregation)), aggregation = 'bagging'; end
if ((nargin < 2) | isempty(aggrParams)), aggrParams = [10 1]; end
if ((nargin < 3) | isempty(classid)), error('ClassID parameter is required!'); end
if ((nargin < 4) | isempty(inputsfile)), 
    inputsfile = 'C:\Documents and Settings\zbarutcu\My Documents\Machine Learning\Olga\Data\ALL_inputs.mat';
    if ~exist(inputsfile, 'file'), inputsfile = 'Data/ALL_inputs.mat'; end
    if ~exist(inputsfile, 'file'), error('Default inputs file not found!'); end
end
if ((nargin < 5) | isempty(labelspath)), 
    labelspath = 'C:/Documents and Settings/zbarutcu/My Documents/Machine Learning/Olga/Data/outputs_%s.mat.txt';
    if ~exist(sprintf(labelspath, classid), 'file'), labelspath = 'Data/outputs_%s.mat.txt'; end
    if ~exist(sprintf(labelspath, classid), 'file'), error('Default labels path not found!'); end
end
if ((nargin < 6) | isempty(resultsdir)), 
    resultsdir = pwd;
end
if ((nargin < 7) | isempty(kerneltype)), 
    kerneltype = 'linear';
end

if ~exist(resultsdir, 'dir'), error('Results dir not found!'); end
if ~ischar(inputsfile), error('Inputs file name must be string!'); end

Labels = [];
labelscolindex = [];
if iscell(labelspath) % <labelspath> can be {<Labels.mat_path>, <colindex>} or {<Labels>, <colindex>}
    labelscolindex = labelspath{2};
    labelspath = labelspath{1};
end
if ~ischar(labelspath)
    Labels = labelspath;
end

% set parameters
AlgName = 'SVMlight';
AlgParams = struct;
AlgParams.KernelType = kerneltype;
%AlgParams.C = 'auto';
%AlgParams.C = Inf;
AlgParams.C = '';
AlgParams.KernelParam = 'auto';
AlgParams.Epsilon = 1e-6;
AlgParams.NormalizeInputs = 'no'; % we will pre-normalize

verbose = 2;

% determine results file name
ResultsFile = [resultsdir, '/', classid, '_',AlgName,'_', ...
        aggregation, num2str(aggrParams(1)), 'x', num2str(aggrParams(2)), ...
        '_', AlgParams.KernelType, ...
        '_C',num2str(AlgParams.C)];
switch AlgParams.KernelType
    case 'rbf'
        ResultsFile = [ResultsFile, '_G', num2str(AlgParams.KernelParam)];
end
ResultsFile = [ResultsFile, '.res.mat'];

% check existing results
if exist(ResultsFile, 'file')
    disp([ResultsFile, ' exists, skipped!']);
    Results = struct;
    return
end

% check flag file (another process working)
TempFile = [ResultsFile, '.tmp'];
if exist(TempFile, 'file')
    disp([ResultsFile, ' in progress, skipped!']);
    Results = struct;
    return
end
StartTime = clock;
save('-ASCII', TempFile, 'StartTime');

disp(['InputsFile: ', inputsfile]);

if isempty(Labels)
    labelsfile = sprintf(labelspath, classid);
    disp(['LabelsFile: ', labelsfile]);
else
    disp(['LabelsFile: <data given>']);
end

% load inputs and labels
if (isequal(inputsfile(end-3:end), '.mat'))
    Inputs = load(inputsfile);
    Inputs = Inputs.ALL_inputs;
else
    Inputs = spconvert(dataread('file', inputsfile, '', -1,'delimiter','\t'));
    %Inputs = spconvert(load('-ASCII', inputsfile));
end

% load labels
if isempty(Labels)
    if isequal(labelsfile(end-3:end), '.mat')
        Labels = load(labelsfile);
        Labels = Labels.Labels;
    else
        Labels = spconvert(dataread('file', labelsfile, '', -1,'delimiter','\t'));
    end
end
if ~isempty(labelscolindex)
    Labels = Labels(:, labelscolindex);
end

%% remove index row and column
%Inputs = Inputs(2:end,2:end);
% REUTERS: Only remove index column
Inputs = Inputs(:,2:end);

%% N2: remove rows where the label is NaN (used for removing ancestor positives)
disp('N2: NOT using NaN labels!');
I_rows = find(~isnan(Labels));
Labels = Labels(I_rows,:);
Inputs = Inputs(I_rows,:);

%% N1: convert NaN labels to 0 (use ancestor positives as negatives)
%disp('N1: USING NaN labels as negatives!');
%I_rows = find(isnan(Labels));
%Labels(I_rows,:) = 0;
%I_rows = (1:size(Labels,1))';

disp([classid, ' started...']);

%% run experiment with pre-normalization and parameter selection
%if isequal(AlgParams.C, 'auto') | (~isequal(AlgParams.KernelType, 'linear') & isequal(AlgParams.KernelParam, 'auto'))
%    Results = exp_run(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'normalize', 'select');
%else
%    Results = exp_run(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'normalize');
%end

% REUTERS: run experiment HARDCODED, WITHOUT pre-normalization and parameter selection
if isequal(AlgParams.C, 'auto') | (~isequal(AlgParams.KernelType, 'linear') & isequal(AlgParams.KernelParam, 'auto'))
    Results = exp_run_SVMlight(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'no', 'select');
else
    Results = exp_run_SVMlight(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'no');
end

Results.I_rows = I_rows;
Results.ClassID = classid;

save(ResultsFile, 'Results');

delete(TempFile);

disp([classid, ' completed!']);
