function Results = exp_kNN(aggregation, aggrParams, classid, inputsfile, labelspath, resultsdir, k)
% function Results = exp_kNN(aggregation, aggrParams, classid, inputsfile, labelspath, resultsdir, k)
%
% <labelspath> can be one of:
%   <path_template>
%   <preloaded_matrix>
%   {<path_template>, <colindex>}
%   {<preloaded_matrix>, <colindex>}

% validate args
if nargin < 1, aggregation = 'loo'; end;
if nargin < 2, aggrParams = []; end;
if ((nargin < 3) | isempty(classid)), classid = 'human'; end
if ((nargin < 4) | isempty(inputsfile)), 
    inputsfile = 'C:\Documents and Settings\Administrator\My Documents\Machine Learning\Shape\Data\ALL_inputs.mat';
    if ~exist(inputsfile, 'file'), inputsfile = 'Data/ALL_inputs.mat'; end
    if ~exist(inputsfile, 'file'), error('Default inputs file not found!'); end
end
if ((nargin < 5) | isempty(labelspath)), 
    labelspath = 'C:/Documents and Settings/Administrator/My Documents/Machine Learning/Shape/Data/outputs/outputs_%s.mat.txt';
    %labelspath = 'C:/Documents and Settings/Administrator/My Documents/Machine Learning/Shape/Data/Labels.mat';
    if ~exist(sprintf(labelspath, classid), 'file'), labelspath = 'Data/outputs_%s.mat.txt'; end
    if ~exist(sprintf(labelspath, classid), 'file'), error('Default labels path not found!'); end
end
if ((nargin < 6) | isempty(resultsdir)), 
    resultsdir = pwd;
end
if ((nargin < 7) | isempty(k)), 
    k = 'auto';
end

if ~exist(resultsdir, 'dir'), error('Results dir not found!'); end
if ~ischar(inputsfile), error('Inputs file name must be string!'); end

Labels = [];
labelscolindex = [];
if iscell(labelspath) % <labelspath> can be {<Labels.mat_path>, <colindex>} or {<Labels>, <colindex>}
    labelscolindex = labelspath{2};
    labelspath = labelspath{1};
end
if ~ischar(labelspath)
    Labels = labelspath;
end

% set parameters
AlgName = 'kNN';
AlgParams = struct;
AlgParams.k = k;
AlgParams.NormalizeInputs = 'no'; % we will pre-normalize (or not normalize, in this case)

verbose = 2;

% determine results file name
if isequal(aggregation, 'loo')
    aggrparamtext = '';
else
    aggrparamtext = [num2str(aggrParams(1)), 'x', num2str(aggrParams(2))];
end
ResultsFile = [resultsdir, '/', classid,'_',AlgName,'_', ...
        aggregation, aggrparamtext, ...
        '_k',num2str(AlgParams.k)];
ResultsFile = [ResultsFile, '.res.mat'];

% check existing results
if exist(ResultsFile, 'file')
    disp([ResultsFile, ' exists, skipped!']);
    Results = struct;
    return
end

% check flag file (another process working)
TempFile = [ResultsFile, '.tmp'];
if exist(TempFile, 'file')
    disp([ResultsFile, ' in progress, skipped!']);
    Results = struct;
    return
end
StartTime = clock;
save('-ASCII', TempFile, 'StartTime');


disp(['InputsFile: ', inputsfile]);

if isempty(Labels)
    labelsfile = sprintf(labelspath, classid);
    disp(['LabelsFile: ', labelsfile]);
else
    disp(['LabelsFile: <data given>']);
end

% load inputs
if (isequal(inputsfile(end-3:end), '.mat'))
    Inputs = load(inputsfile);
    Inputs = Inputs.ALL_inputs;
else
    Inputs = spconvert(dataread('file', inputsfile, '', -1,'delimiter','\t'));
    %Inputs = spconvert(load('-ASCII', inputsfile));
end

% load labels
if isempty(Labels)
    if isequal(labelsfile(end-3:end), '.mat')
        Labels = load(labelsfile);
        Labels = Labels.Labels;
    else
        Labels = spconvert(dataread('file', labelsfile, '', -1,'delimiter','\t'));
    end
end
if ~isempty(labelscolindex)
    Labels = Labels(:, labelscolindex);
end
%Labels = spconvert(load('-ASCII', labelsfile));

% remove index row and column
Inputs = Inputs(2:end,2:end);

%% N2: remove rows where the label is NaN (used for removing ancestor positives)
disp('N2: NOT using NaN labels!');
I_rows = find(~isnan(Labels));
Labels = Labels(I_rows,:);
Inputs = Inputs(I_rows,:);

%% N1: convert NaN labels to 0 (use ancestor positives as negatives)
%disp('N1: USING NaN labels as negatives!');
%I_rows = find(isnan(Labels));
%Labels(I_rows,:) = 0;
%I_rows = (1:size(Labels,1))';

disp([classid, ' started...']);

% run experiment with pre-normalization and parameter selection
if isequal(AlgParams.k, 'auto')
    % Results = exp_run(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'normalize', 'select');
    Results = exp_run(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'no', 'select');
else
    %Results = exp_run(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'normalize');
    Results = exp_run(AlgName, AlgParams, Inputs, Labels, aggregation, aggrParams, verbose, 'no');
end
Results.I_rows = I_rows;
Results.ClassID = classid;

% Don't save training data
for i=1:length(Results.Folds)
  Results.Folds{i}.ModelParams.StoredInputs = 'Not Saved';
  Results.Folds{i}.ModelParams.StoredLabels = 'Not Saved';
end

save(ResultsFile, 'Results');

delete(TempFile);

disp([classid, ' completed!']);
