function pred_SVMlight(resultsfiles, inputsfile, unknownsfile, outdir)
% function pred_SVMlight(resultsfiles, inputsfile, unknownsfile, outdir)

funcEval = 'alg_SVMlightRaw_eval';

% validate args
if ((nargin < 1) | isempty(resultsfiles)),
    resultsfiles = '~/research/function/rawresults/svml_lin_Cinf_V6/*_SVMlight_bagging10x1_linear_CInf.res.mat';
    % error('Results file(s) not specified!');
end
if ((nargin < 2) | isempty(inputsfile)), 
    inputsfile = '~/research/function/data/ALL_inputs.mat';
    if ~exist(inputsfile, 'file'), inputsfile = 'C:\Documents and Settings\zbarutcu\My Documents\Machine Learning\Olga\Data\ALL_inputs.mat'; end
    if ~exist(inputsfile, 'file'), error('Default inputs file not found!'); end
end
if ((nargin < 3) | isempty(unknownsfile)), 
    unknownsfile = '~/research/function/data/ALL_unknowns.mat';
    if ~exist(unknownsfile, 'file'), unknownsfile = 'C:\Documents and Settings\zbarutcu\My Documents\Machine Learning\Olga\Data\ALL_unknowns.mat'; end
    if ~exist(unknownsfile, 'file'), error('Default unknowns file not found!'); end
end
if ((nargin < 4) | isempty(outdir)), 
    outdir = '~/research/function/rawresults/svml_lin_Cinf_V6_pred';
    % outdir = pwd;
end

if ~exist(outdir, 'dir'), error('Outputs dir not found!'); end
if ~ischar(inputsfile), error('Inputs file name must be string!'); end
if ~ischar(unknownsfile), error('Unknowns file name must be string!'); end

disp(['InputsFile: ', inputsfile]);
disp(['UnknownsFile: ', unknownsfile]);

verbose = 2;

%
% LOAD DATA
%

% load inputs
if (isequal(inputsfile(end-3:end), '.mat'))
    Inputs = load(inputsfile);
    Inputs = Inputs.ALL_inputs;
else
    Inputs = spconvert(dataread('file', inputsfile, '', -1,'delimiter','\t'));
    %Inputs = spconvert(load('-ASCII', inputsfile));
end

% load inputs and labels
if (isequal(unknownsfile(end-3:end), '.mat'))
    Unknowns = load(unknownsfile);
    Unknowns = Unknowns.ALL_unknowns;
else
    Unknowns = spconvert(dataread('file', unknownsfile, '', -1,'delimiter','\t'));
    %Inputs = spconvert(load('-ASCII', inputsfile));
end
% remove index row and column
Inputs = Inputs(2:end,2:end);
Unknowns = Unknowns(2:end,2:end);

%
% PROCESS ALL FILES
%

[resultsdir, thename, theext, thever] = fileparts(resultsfiles);
files = dir(resultsfiles);
for k = 1:size(files,1)
    OutFile = fullfile(outdir, files(k).name)
    resultsfile = fullfile(resultsdir, files(k).name)
    
    % check existing results
    if exist(OutFile, 'file')
        disp([OutFile, ' exists, skipped!']);
        Results = struct;
        continue;
    end
    
    % check flag file (another process working)
    TempFile = [OutFile, '.tmp'];
    if exist(TempFile, 'file')
        disp([OutFile, ' in progress, skipped!']);
        Results = struct;
        continue;
    end
    StartTime = clock;
    save('-ASCII', TempFile, 'StartTime');
    
    % load results (containing model parameters)
    Results = load(resultsfile);
    Results = Results.Results;
    
    %
    % PREDICT HELD-OUTS
    %
    
    N = size(Inputs,1);
    
    MeanY = zeros(N, 1);
    MedianY = zeros(N, 1);
    
    % evaluate all folds on all examples
    Nfolds = size(Results.Folds,1) * size(Results.Folds,2);
    Y = zeros(N, Nfolds);
    for j=1:Nfolds
        Y(:, j) = feval(funcEval, Results.Folds{j}.ModelParams, full(Inputs(:, Results.I_cols)));
    end
    
    % for each example, find held-out folds
    for i=1:N
        P = [];
        for j=1:Nfolds
            fold = Results.Folds{j};
            if ~ismember(i, Results.I_rows(fold.ITrain))
                P = [P, Y(i,j)];
            end
        end
        if ~isempty(P)
            MeanY(i,1) = mean(P,2);
            MedianY(i,1) = median(P,2);
        else
            MeanY(i,1) = NaN;
            MedianY(i,1) = NaN;
        end
    end
    Results.HeldOutMeanOutputsRaw = MeanY(Results.I_rows);
    Results.HeldOutMedianOutputsRaw = MedianY(Results.I_rows);
    
    % these are the removed negatives, with NaN labels during training.
    I_excludedRows = setdiff(1:N, Results.I_rows);
    Results.I_excludedRows = I_excludedRows;
    Results.ExcludedMeanOutputsRaw = MeanY(I_excludedRows);
    Results.ExcludedMedianOutputsRaw = MedianY(I_excludedRows);
    
    %
    % PREDICT UNKNOWNS
    %
    
    N = size(Unknowns,1);
    
    % evaluate all folds on all examples
    Nfolds = size(Results.Folds,1) * size(Results.Folds,2);
    Y = zeros(N, Nfolds);
    for j=1:Nfolds
        Y(:, j) = feval(funcEval, Results.Folds{j}.ModelParams, full(Unknowns(:, Results.I_cols)));
    end
    
    Results.UnknownOutputsRaw = Y;
    Results.UnknownMeanOutputsRaw = mean(Y,2);
    Results.UnknownMedianOutputsRaw = median(Y,2);
    
    %
    % SAVE
    %
    
    
    save(OutFile, 'Results');
    
    delete(TempFile);
    
    disp(['Saved ', OutFile]);
    
end