function pred_chunk(funcEval, ResultsPattern, InputsFile, PredsFile)

% Check for existing Predictions file.
if exist(PredsFile, 'file')
    disp([PredsFile, ' exists, skipped!']);
    return;
end

% check tmp file (another process working).
TempFile = [PredsFile, '.tmp'];
if exist(TempFile, 'file')
    disp([PredsFile, ' already in progress, skipped!']);
    return;
end

% Indicate that prediction is in progress: Write Outputs tmp file.
StartTime = clock;
save('-ASCII', TempFile, 'StartTime');

% Load Inputs file.
Inputs = load(InputsFile);
Inputs = Inputs.ALL_inputs;

% Get class IDs.
ClassIDs = getSelectedClassIDs;

% Prepare results struct.
numInputs = size(Inputs, 1);
numClasses = length(ClassIDs);
Predictions = struct;
Predictions.MeanY = zeros(numInputs, numClasses);
Predictions.MedianY = zeros(numInputs, numClasses);

% Predict all examples for each class.
for k = 1:numClasses
  % Load Results file for the class (containing model parameters).
  classID = ClassIDs{k};
  ResultsFile = sprintf(ResultsPattern, classID);
  Results = load(ResultsFile);
  Results = Results.Results;

  % Evaluate all folds on all examples.
  Nfolds = size(Results.Folds,1) * size(Results.Folds,2);
  Y = zeros(numInputs, Nfolds);
  for j=1:Nfolds
      Y(:, j) = feval(funcEval, Results.Folds{j}.ModelParams, Inputs(:, Results.I_cols));
  end
  Predictions.MeanY(:,k) = mean(Y,2);
  Predictions.MedianY(:,k) = median(Y,2);
end

save(PredsFile, 'Predictions');

delete(TempFile);

disp([PredsFile, ' done.']);
