function [MeanY, MedianY] = predictTestExamples(funcEval, Inputs, I_ex, ResPattern, ClassIDList)
% function [MeanY, MedianY] = predictTestExamples(funcEval, Inputs, I_ex, ResPattern, ClassIDList)
%
% Makes combined predictions for given unknown genes.
%
% Inputs: raw inputs WITHOUT index row or col (e.g. ALL_unknowns(2:end, 2:end)
% I_ex: col vector of row indices, default is all rows.
% MeanY, MedianY are NxK for N genes (Inputs rows) and K classes (ResArray length).

if nargin < 5
    ClassIDList = getSelectedClassIDs;
end

if nargin < 4
    ResPattern = 'C:/My Projects/Matlab/Results/SVMlight2_full/%s_SVMlight_bagging10x1_linear_Cauto.res.mat';
end

if nargin < 3, 
    I_ex = [];
end

if isempty(I_ex)
    I_ex = (1:size(Inputs,1))'; %'
end

N = size(I_ex,1);
K = length(ClassIDList);

MeanY = zeros(N, K);
MedianY = zeros(N, K);

% TODO: Load Inputs file
if ischar(Inputs)
  Inputs = load(Inputs);
  Inputs = Inputs.ALL_inputs;
end

% for each class
for k=1:K
  classid = ClassIDList{k};
  disp(['Evaluating ', classid, ' (', num2str(k), ' of ', num2str(K), ')']);
  
  %
  % Check that this class hasn't been predicted: Outputs tmp or mat file doesn't exist
  %

  % determine results file name
  ResultsFile = sprintf(respattern, classid);
  OutputsFile = sprintf(outpattern, classid);
  
  % check existing outputs
  if exist(OutputsFile, 'file')
      disp([OutputsFile, ' exists, skipped!']);
      continue;
  end
  
  % check tmp file (another process working)
  TempFile = [OutputsFile, '.tmp'];
  if exist(TempFile, 'file')
      disp([OutputsFile, ' in progress, skipped!']);
      continue;
  end

  % Indicate that prediction is in progress: Write Outputs tmp file
  StartTime = clock;
  save('-ASCII', TempFile, 'StartTime');

  % Load Results file (containing model parameters)
  Results = load(ResultsFile);
  Results = Results.Results;

  % TODO: Run all folds on Inputs
  % TODO: Write Outputs mat file
  % TODO: Delete Outputs tmp file
    
    
    
    % evaluate all folds on all examples
    Nfolds = size(Results.Folds,1) * size(Results.Folds,2);
    Y = zeros(N, Nfolds);
    for j=1:Nfolds
        Y(:, j) = feval(funcEval, Results.Folds{j}.ModelParams, Inputs(I_ex, Results.I_cols));
    end
    MeanY(:,k) = mean(Y,2);
    MedianY(:,k) = median(Y,2);
end
