function [MeanY, MedianY] = predictUnknowns(funcEval, Inputs, I_ex, ResPattern, GOIDList)
% function [MeanY, MedianY] = predictUnknowns(funcEval, Inputs, I_ex, ResPattern, GOIDList)
%
% Makes combined predictions for given unknown genes.
%
% Inputs: raw inputs WITHOUT index row or col (e.g. ALL_unknowns(2:end, 2:end)
% I_ex: col vector of row indices, default is all rows.
% MeanY, MedianY are NxK for N genes (Inputs rows) and K classes (ResArray length).

if nargin < 5
    GOIDList = getSelectedGOIDs;
end

if nargin < 4
    ResPattern = 'C:/My Projects/Matlab/Results/SVMlight2_full/%s_SVMlight_bagging10x1_linear_Cauto.res.mat';
end

if nargin < 3, 
    I_ex = [];
end

if isempty(I_ex)
    I_ex = (1:size(Inputs,1))'; 
end

N = size(I_ex,1);
K = length(GOIDList);

MeanY = zeros(N, K);
MedianY = zeros(N, K);

% for each class
for k=1:K
    goid = GOIDList{k};
    disp(['Evaluating ', goid, ' (', num2str(k), ' of ', num2str(K), ')']);
    
    % load results (containing model parameters)
    Results = load(sprintf(ResPattern, goid(4:end)));
    Results = Results.Results;

    % evaluate all folds on all examples
    Nfolds = size(Results.Folds,1) * size(Results.Folds,2);
    Y = zeros(N, Nfolds);
    for j=1:Nfolds
        Y(:, j) = feval(funcEval, Results.Folds{j}.ModelParams, Inputs(I_ex, Results.I_cols));
    end
    MeanY(:,k) = mean(Y,2);
    MedianY(:,k) = median(Y,2);
end
