function createReutersSynthDatasets(dataroot)

if nargin < 1
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data';
end

rows = 10000;
classes = 101;

% Load REUTERS hierarchy.
ClassIDs = getSelectedClassIDs; % Reuters.
NodeParents = loadNodeParents('/n/fs/grad/zbarutcu/research/function/Reuters/data/nodeParents.mat.txt', ClassIDs);

for prevalence = [.01, .05, .10, .50]
  for recall = [.10, .30, .50, .70, .90]
    for specificity=[.95, .75, .55, .35, .15]
      if recall < 1-specificity, continue; end % Use upper triangular part of ROC.
      
      % Make dataset name.
      datasetName = sprintf('hReuters_p%02.0f_r%02.0f_s%02.0f', prevalence * 100, recall * 100, specificity * 100);
      disp(sprintf('Generating dataset "%s"...', datasetName));
      
      % Create random data.
      [Labels, Outputs] = createSyntheticData(NodeParents, rows, classes, prevalence, specificity, recall);
      
      % Create dataset dir if it doesn't exist.
      datasetDir = [dataroot, '/', datasetName];
      if ~exist(datasetDir, 'dir')
        mkdir(datasetDir);
      end

      % Save data files.
      save([datasetDir, '/Labels.mat'], 'Labels');
      save([datasetDir, '/Outputs.mat'], 'Outputs');
      
      % Create and save Bayes net.
      bnet = combineBayesian_discrete(Labels, Outputs, NodeParents, ClassIDs, 'down');
      save([datasetDir, '/bnet_down.mat'], 'bnet');

      % Create chunks dir if it doesn't exist.
      chunksDir = [datasetDir, '/chunks'];
      if ~exist(chunksDir, 'dir')
        mkdir(chunksDir);
      end
      
      % Chunk Outputs.
      chunkData(1000, ...
        {[datasetDir, '/Outputs.mat']}, ...
        [chunksDir, '/Outputs.%03d.chunk.mat'], ...
        'Outputs');
    end
  end
end
