function createSynthHierarchyDatasets_patch(hierarchyNames, dataroot)

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
    'NT5D3'
    'NT5D4'
    'RT4D5'
    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);

  % Make up class IDs.
  ClassIDs = cell(hier.NumClasses, 1);
  for i=1:length(ClassIDs)
    ClassIDs{i} = sprintf('C%d', i);
  end
    
  for prevalence = [.01, .05, .10, .25, .50]
    for specificity = [.10, .30, .50, .70, .90]
      for recall = [.15, .35, .55, .75, .95]
        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('p%02.0f_s%02.0f_r%02.0f', prevalence * 100, specificity * 100, recall * 100);
        disp(sprintf('Patching dataset %s/%s...', hierName, datasetName));
        
        datasetDir = [hierDir, '/', datasetName];
  
        % LOAD data files.
        Labels = load([datasetDir, '/Labels.mat']);
        Labels = Labels.Labels;
        Outputs = load([datasetDir, '/Outputs.mat']);
        Outputs = Outputs.Outputs;
        
        % Create and save Bayes nets.
        %bnet = combineBayesian_discrete(Labels, Outputs, hier.NodeParents, ClassIDs, 'down');
        %save([datasetDir, '/bnet_down.mat'], 'bnet');
        bnet = combineBayesian_discrete(Labels, Outputs, hier.NodeParents, ClassIDs, 'none');
        save([datasetDir, '/bnet_none.mat'], 'bnet');
        %bnet = combineBayesian_discrete(Labels, Outputs, hier.NodeParents, ClassIDs, 'up');
        %save([datasetDir, '/bnet_up.mat'], 'bnet');        
      end
    end
  end
end
