function createSynthHierarchyDatasets(hierarchyNames, dataroot, rows)

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1
  hierarchyNames = {
    'T2D4'
    'T2D6'
    'T5D2'
    'T5D3'
%    'NT5D3'
%    'NT5D4'
%    'RT4D5'
%    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

if nargin < 3
  rows = 10000;
end

for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);

  % Make up class IDs.
  ClassIDs = cell(hier.NumClasses, 1);
  for i=1:length(ClassIDs)
    ClassIDs{i} = sprintf('C%d', i);
  end
  
%  for prevalence = [.50] % [.01, .05, .10, .25, .50]
%    for specificity = [.99] % [.10, .30, .50, .70, .90]
%      for recall = .35 : .10 : .95 % [.15, .35, .55, .75, .95]
  for prevalence = [.75] % [.01, .05, .10, .25, .50]
    for accuracy = [.70, .80, .90]
        specificity = accuracy;
        recall = accuracy;

        % Use upper-triangular half of ROC space
        if recall <= 1 - specificity, continue; end
      
        % Make dataset name.
        datasetName = sprintf('q%02.0f_a%02.0f', prevalence * 100, accuracy * 100);
        disp(sprintf('Generating dataset %s for %s...', datasetName, hierName));
        
        % Create random data.
        [Labels, Outputs] = createSyntheticData_topdown(hier.NodeParents, rows, hier.NumClasses, prevalence, specificity, recall);
        
        % Create dataset dir if it doesn't exist.
        datasetDir = [hierDir, '/', datasetName];
        if ~exist(datasetDir, 'dir')
          mkdir(datasetDir);
        end
  
        % Save data files.
        save([datasetDir, '/Labels.mat'], 'Labels');
        save([datasetDir, '/Outputs.mat'], 'Outputs');
        
        % Create and save Bayes nets.
        bnet = combineBayesian_discrete(Labels, Outputs, hier.NodeParents, ClassIDs, 'down');
        save([datasetDir, '/bnet_down.mat'], 'bnet');
        bnet = combineBayesian_discrete(Labels, Outputs, hier.NodeParents, ClassIDs, 'none');
        save([datasetDir, '/bnet_none.mat'], 'bnet');
        bnet = combineBayesian_discrete(Labels, Outputs, hier.NodeParents, ClassIDs, 'up');
        save([datasetDir, '/bnet_up.mat'], 'bnet');
  
        % Create chunks dir if it doesn't exist.
        chunksDir = [datasetDir, '/chunks'];
        if ~exist(chunksDir, 'dir')
          mkdir(chunksDir);
        end
        
        % Chunk Outputs.
        chunkData(1000, ...
          {[datasetDir, '/Outputs.mat']}, ...
          [chunksDir, '/Outputs.%03d.chunk.mat'], ...
          'Outputs');
        
      end
    %end
  end
end
