function [Labels, Outputs] = createSyntheticData(NodeParents, rows, classes, prevalence, specificity, recall, precision)
% function Data = createSyntheticData(size, prevalence, specificity, recall)

if nargin < 7
  precision = [];
end

if ~isempty(precision) && isempty(specificity)
  FP = recall * prevalence * (1 - precision) / precision;
  specificity = 1 - (FP / (1 - prevalence));
end
      
% Roll dice for Labels.
R = rand(rows, classes);
Labels = R < prevalence;

% Make data hierarchically consistent by propagating labels upwards.
Labels = tmp_propagateLabels(Labels, 'up', [], NodeParents);

% Roll dice for Outputs.
R = rand(rows, classes);
% if Labels==1: P(1) = recall
% if Labels==0: P(0) = specificity
Outputs = (Labels & (R < recall)) | (~Labels & (R > specificity));
