function [Labels, Outputs] = createSyntheticData(NodeParents, rows, classes, prevalence, specificity, recall, precision)
% function Data = createSyntheticData(size, prevalence, specificity, recall)

if nargin < 7
  precision = [];
end

% Either precision or specificity must be given, but not both.
assert(xor(isempty(specificity), isempty(precision)));

% Roll dice for Labels.
R = rand(rows, classes);
Labels = R < prevalence;

% Make data hierarchically consistent by propagating labels upwards.
Labels = tmp_propagateLabels(Labels, 'up', [], NodeParents);

% Roll dice for Outputs.
Outputs = zeros(size(Labels));
for i=1:classes
  % Need to recompute specificity from precision, since prevalence changes by
  % propagating labels.
  if ~isempty(precision)
    newprevalence = sum(Labels(:,i), 1) / rows;
    FP = recall * newprevalence * (1 - precision) / precision; % TP * (FP)/(TP)
    if newprevalence < 1
      specificity = 1 - (FP / (1 - newprevalence)); % TN / (TN+FP)
    else
      specificity = 0; % never gonna be used if there are no negatives.
    end
  end

  % if Labels==1: P(1) = recall
  % if Labels==0: P(0) = specificity
  R = rand(rows, 1);
  Outputs(:,i) = (Labels(:,i) & (R < recall)) | (~Labels(:,i) & (R > specificity));
end
