function testPlotAUC

if nargin < 2, dataroot = []; end
if isempty(dataroot)
  dataroot = '/n/fs/grad/zbarutcu/research/function/Synth/data/hierarchies';
end

if nargin < 1, hierarchyNames = {}; end
if isempty(hierarchyNames)
  hierarchyNames = {
    'T2D4'
    'T2D6'
  %  'T5D2'
  %  'T5D3'
%    'NT5D3'
%    'NT5D4'
%    'RT4D5'
%    'RT5D4'
  };
end
if ischar(hierarchyNames), hierarchyNames = {hierarchyNames}; end

for iHier=1:length(hierarchyNames)
  hierName = hierarchyNames{iHier};
  hierDir = [dataroot, '/', hierName];
  hierFile = [hierDir, '/Hierarchy.mat'];
  hier = load(hierFile);
  classes = hier.NumClasses;

  for prevalence = [.75] % [.01, .05, .10, .25, .50]
    for accuracy = .90 % [.70, .80, .90]
      specificity = accuracy;
      recall = accuracy;
      % Use upper-triangular half of ROC space
      if recall <= 1 - specificity, continue; end
    
      % Make dataset name.
      datasetName = sprintf('q%02.0f_a%02.0f', prevalence * 100, accuracy * 100);
      %disp(sprintf('Generating dataset %s for %s...', datasetName, hierName));
      
      datasetDir = [hierDir, '/', datasetName];

      figure;
      axis([0 0.2 0.8 1]);
      axis square;
      hold on;
      t=sprintf('%s: q=%02.0f%% ; a=%02.0f%%', hierName, prevalence * 100, accuracy * 100);
      title(t);
      xlabel('1-Specificity');
      ylabel('Sensitivity');
      plot([0 1], [0 1], 'k--');
      C0 = load([datasetDir, '/Confusions.mat']);
      Cup = load([datasetDir, '/Confusions_bnet_up.mat']);
      Cdown = load([datasetDir, '/Confusions_bnet_down.mat']);
      Cnone = load([datasetDir, '/Confusions_bnet_none.mat']);
      for i = 1:length(C0.C)
        CS0 = GetConfusionStats(C0.C{i});
        CSup = GetConfusionStats(Cup.C{i});
        CSdown = GetConfusionStats(Cdown.C{i});
        CSnone = GetConfusionStats(Cnone.C{i});
      
        mecNone = [.5 .5 .5];
        mecUp = 'blue';
        mecDown = 'green';
      
        %field = 'FMeasure';
        %field = 'Accuracy';
        %field = 'AUC';
      
        Hraw  = plot(1-CS0.Specificity, CS0.Recall, 'k.');
 %       Hnone = plot(1-CSnone.Specificity, CSnone.Recall, 'o', 'MarkerEdgeColor', mecNone);
 %       Hup   = plot(1-CSup.Specificity, CSup.Recall, 'o', 'MarkerEdgeColor', mecUp);
        Hdown = plot(1-CSdown.Specificity, CSdown.Recall, 'k*'); %, 'MarkerEdgeColor', mecDown);
      
        %if CS0.Accuracy >= CS0.Prevalence && CS0.Accuracy >= 1 - CS0.Prevalence,
          %set(Hnone, 'MarkerFaceColor', mecNone);
        %  set(Hup, 'MarkerFaceColor', mecUp);
        %  set(Hdown, 'MarkerFaceColor', mecDown);
        %end
      
        %plot(CS0.FMeasure, CS1.FMeasure, 'ko', 'MarkerFaceColor', mfc);
        %plot(CS0.Precision, CS1.Precision, 'ko', 'MarkerFaceColor', mfc);
        %plot(CS0.AUC, CS1.AUC, 'ko', 'MarkerFaceColor', mfc);
      end

    end
  end
end
