function tmp_writeAUCChanges(Labels, OldOutputs, NewOutputs, ClassIDList)
% function tmp_writeAUCChanges((Labels, OldOutputs, NewOutputs, ClassIDList)
%
% Writes out the changes in AUCs.
%
% <Labels> is discrete training labels.
% <OldOutputs> is continuous classifier outputs (NaN where no output)
% <NewOutputs> is improved continuous classifier outputs

if nargin < 4, ClassIDList = []; end;

allClassIDs = getSelectedClassIDs;
if isempty(ClassIDList)
    ClassIDList = allClassIDs;
end

if nargin < 3
    error('You must specify 3 parameters');
end

K = length(ClassIDList);

% for each class
for k=1:K
    ClassID = ClassIDList{k};

	j = find(ismember(allClassIDs, ClassID));
	if isempty(j)
		warning(['ClassID ', ClassID, ' not found in selected ClassIDs!']);
		continue;
	end
	    
    % write AUC changes
    I_rows = find(~isnan(OldOutputs(:, j)));
    thisOldOut = OldOutputs(I_rows, j);
    thisNewOut = NewOutputs(I_rows, j);
    thisLabels = Labels(I_rows, j);
    
    oldAUC = GetROC(thisLabels, thisOldOut);
    newAUC = GetROC(thisLabels, thisNewOut);
    
    disp([ClassID, sprintf(' %g', [oldAUC, newAUC, newAUC - oldAUC])]);
end
