"""
Exports all datasets to Matlab in a single file.
"""
import os

#dataRoot = 'C:\\Documents and Settings\\zbarutcu\\My Documents\\Machine Learning\\Olga\\Data\\'
dataRoot = os.getenv('HOME') + '/research/function/data/'

geneIndexFileName = 'geneNameIndex.txt'
inputsFileName = 'ALL_inputs.mat.txt'
##outputsFileName = 'ALL_outputs.mat.txt'
outputsPathTemplate = dataRoot + 'outputs/outputs_%s.mat.txt'
unknownsFileName = 'ALL_unknowns.mat.txt'

from sets import Set

# reset timer
from time import clock
t0 = clock() ; t1 = t0
def printTime():
    global t0, t1
    t2 = clock() ; print "--TIME step=%d ; total=%d" % (t2-t1, t2-t0) ; t1 = t2

 
print 'Loading gene index...'
import GeneIndex
geneIndex = GeneIndex.GeneIndex()
geneIndex.loadFile()
geneIndex.printSummary()
printTime()

print 'Loading GO tree...'
import ontology
go = ontology.Ontology()
go.loadFile()
printTime()
print 'Loading GO annotations...'
import association
assoc = association.Association(translator = geneIndex.Any2ORF)
assoc.loadFile()
go.addAnnotations(assoc)
go.printSummary()
printTime()

print 'Loading GRID...'
import GRID
grid = GRID.Interactions(translator = geneIndex.Any2ORF)
grid.loadFile()
grid.printSummary()
printTime()

print 'Loading microarrays...'
import MicroarrayCollection
mc = MicroarrayCollection.MicroarrayCollection()
mc.loadAll()
mc.printSummary()
commonORFs = mc.getCommonORFs()
print 'Microarray common genes: %d' % len(commonORFs)
commonORFs = commonORFs.intersection(Set(grid.genes))
print 'Microarray+GRID common genes: %d' % len(commonORFs)
commonORFs = commonORFs.intersection(Set(geneIndex.ORFs))
print 'Microarray+GRID+GeneIndex common genes: %d' % len(commonORFs)
printTime()

print 'Loading Colocalization_complex (from Kara)...'
import Colocalization_complex
complex = Colocalization_complex.Colocalization_complex(geneIndex)
complex.loadFile()
complex.printSummary()
printTime()

print 'Loading Colocalization_L_neg (from Yale)...'
import Colocalization_L_neg
L_neg = Colocalization_L_neg.Colocalization_L_neg()
L_neg.loadFile()
L_neg.printSummary()
printTime()

print 'Loading TF Sites...'
import TFSites
tfsites = TFSites.TFSites(minGeneCount = 3)
tfsites.loadFile()
tfsites.printSummary()
printTime()

print 'Determining ORF lists...'
unknownORFs = [g['ORF'] for g in go.nodesByID['GO:0000004'].genes if g['ORF'] in geneIndex.ORFs] # 'GO:0000004' is 'biological_process unknown'
trainingORFs = [x for x in commonORFs if x not in unknownORFs]
unknownORFs = [x for x in unknownORFs if x in commonORFs]
print "Training ORFs: %d" % len(trainingORFs)
print "Unknown  ORFs: %d" % len(unknownORFs)
printTime()

# exporting function, exports all datasets to given OPEN file
def exportAll(targetFile, ORFList):
    global geneIndex, grid, mc, tfsites, L_neg, complex
    colOffset = grid.saveMerged(targetFile, ORFList, grid.genes, geneIndex.ORFs)
    colOffset = mc.exportMerged(targetFile, ORFList, indexList = None, coloffset = colOffset, rowoffset = 1)
    colOffset = tfsites.export(targetFile, ORFList, coloffset = colOffset, rowoffset = 1)
    colOffset = L_neg.export(targetFile, ORFList, coloffset = colOffset, rowoffset = 1)
    colOffset = complex.export(targetFile, ORFList, coloffset = colOffset, rowoffset = 1)

###
### Export inputs
###
##print 'Exporting inputs...'
##inputsFile = file(dataRoot + inputsFileName, 'wt')
##try:
##    exportAll(inputsFile, trainingORFs)
##finally:
##    inputsFile.close()
##printTime()
##
###
### Export unknowns
###
##print 'Exporting unknowns...'
##unknownsFile = file(dataRoot + unknownsFileName, 'wt')
##try:
##    exportAll(unknownsFile, unknownORFs)
##finally:
##    unknownsFile.close()
##printTime()

#
# Export outputs
#
import SelectedGONodes;

print 'Exporting outputs...'
for goid in SelectedGONodes.SelectedGOIDs:
    fname = outputsPathTemplate % goid[3:]
    go.saveLabels(fname, trainingORFs, [goid])
printTime()

print 'Exporting gene index...'
fname = dataRoot + geneIndexFileName;
f = file(fname, 'wt')
try:
    f.writelines([x+'\n' for x in geneIndex.ORFs])
finally:
    f.close()
printTime()

print "Exporting completed!"
