from TextData import TextData
import sets
import os

class Colocalization_complex(TextData):
    def __init__(self, geneIndex):
        #self.DEFAULT_FILE_NAME = r"C:\Documents and Settings\Zafer\My Documents\Machine Learning\Olga\Data\Colocalization\complex_out_edit.txt"
        self.DEFAULT_FILE_NAME = os.getenv('HOME') + "/research/function/rawdata/Colocalization/complex_out_edit.txt"
        self.geneIndex = geneIndex
        if self.geneIndex is None:
            print "*** WARNING: Colocalization_complex has no geneIndex. Using CGN instead of ORF."
        self.clear()

    def clear(self):
        self.groups = {}
        self.geneToGroups = {}
        self.genes = []

    def loadLines(self, lines):
        print " parsing...",
        for line in lines:
            if line.startswith('Component:'):
                cols = line.split('\t')
                groupName = cols[0].strip()
                genes = cols[2].strip().split(' ')
                if self.geneIndex is not None:
                    genes = [self.geneIndex.Any2ORF(cgn) for cgn in genes]
                self.groups[groupName] = genes
                for gene in genes:
                    if gene in self.geneToGroups:
                        self.geneToGroups[gene].add(groupName)
                    else:
                        self.geneToGroups[gene] = sets.Set([groupName])
        self.genes = self.geneToGroups.keys()
        print "OK"

    def export(self, outfile, ORFList, rowoffset = 0, coloffset = 0):
        ftype = type(outfile)
        if ftype is str:
            f = file(outfile, 'wt')
        elif ftype is file:
            f = outfile
        
        try:
            # write each row
            localeLists = self.groups.values()
            for (orf, r) in zip(ORFList, range(0, len(ORFList))):
                # write each col
                for (localeORFs, c) in zip(localeLists, range(0, len(localeLists))):
                    if orf in localeORFs:
                        f.write('%d\t%d\t1\n' % (r+1+rowoffset, c+1+coloffset))
                    
            # ensure matrix size by forcing the last element
            if ORFList[-1] not in localeLists[-1]:
                f.write('%d\t%d\t0\n' % (len(ORFList)+rowoffset, len(localeLists)+coloffset))
            
        finally:
            if ftype is str:
                f.close()
        
        return len(localeLists)+coloffset

    def printSummary(self):
        print "%s instance:" % self.__class__.__name__
        print " Groups: %d" % len(self.groups)
        print " Genes: %d" % len(self.genes)
        counts = map(len, self.groups.values())
        print " Min. size: %d" % min(counts)
        print " Max. size: %d" % max(counts)
        print "(End)"


#complex = Colocalization_complex.Colocalization_complex() ; complex.loadFile()
