"""
Zafer Barutcuoglu 2004/03/04
"""

from TextData import TextData
from Table import Table

import sys
import os

class GeneIndex(TextData):
    """Parses gene name registry data downloaded from:
        ftp://genome-ftp.stanford.edu/pub/yeast/data_download/chromosomal_feature/SGD_features.tab

    self.genes is a table of gene names in various formats.
    """
    def __init__(self):
        self.clear()
        self.DEFAULT_URL = "ftp://genome-ftp.stanford.edu/pub/yeast/data_download/chromosomal_feature/SGD_features.tab"
        #self.DEFAULT_FILE_NAME = r"C:\Documents and Settings\Zafer\My Documents\Machine Learning\Olga\Data\SGD_features.tab.txt"
        self.DEFAULT_FILE_NAME = os.getenv('HOME') + "/research/function/rawdata/SGD_features.tab.txt"

        # old code might need these
        self.OLD_URL = "ftp://genome-ftp.stanford.edu/pub/yeast/data_download/chromosomal_feature/chromosomal_feature.tab"
        self.OLD_FILE_NAME = r"C:\Documents and Settings\Zafer\My Documents\Machine Learning\Olga\Data\chromosomal_feature.tab.txt"
    
    def clear(self):
        self.genes = Table(['CGN', 'aliases', 'ORF', 'SGDID', 'OtherSGDIDs'])

    def loadLines(self, lines):
##        print " parsing...",
        for line in lines:
            cols = line.split('\t')

##            #old code for chromosomal_feature.tab
##            orf = cols[0].strip()
##            cgn = cols[1].strip()
##            if len(cgn) == 0: cgn = None
##            aliases = [a for a in cols[2].strip().split('|') if len(a) > 0]
##            sgdid = cols[8].strip()
##            othersgdids = [id for id in cols[9].strip().split('|') if len(id) > 0]
            
            orf = cols[3].strip()
            if len(orf) ==0: continue
            
            cgn = cols[4].strip()
            if len(cgn) == 0: cgn = None

            aliases = [a for a in cols[5].strip().split('|') if len(a) > 0]
            sgdid = cols[0].strip()
            othersgdids = [id for id in cols[7].strip().split('|') if len(id) > 0]

            self.genes.append([cgn,aliases,orf,sgdid,othersgdids])
##        print "OK"
##        print " indexing...",
        self.genes.indexBy('CGN', isUnique = True)
        self.genes.indexByList('aliases', isUnique = False)
        self.genes.indexBy('ORF', isUnique = True)
        self.genes.indexBy('SGDID', isUnique = True)
        self.genes.indexByList('OtherSGDIDs', isUnique = True)
        self.ORFs = self.genes.indexKeys('ORF')
##        print "OK"
    
##    def ORF2CGN(self, orf):
##        gene = self.genes.find('ORF', orf)
##        if gene is not None:
##            return gene[0]
##        else:
##            return None
##    
##    def CGN2ORF(self, cgn):
##        gene = self.genes.find('CGN', cgn)
##        if gene is not None:
##            return gene[2]
##        else:
##            return None
    
    def Any2ORF(self, name):
        gene = self.genes.find('CGN', name)
        if gene is None:
            gene = self.genes.find('aliases', name)
        if gene is None:
            gene = self.genes.find('SGDID', name)
        if gene is None:
            gene = self.genes.find('ORF', name)
        
        if gene is None: return None
        elif type(gene) is list: # alias matched
            if len(gene) == 1:
                return gene[0]['ORF']
            else:
                sys.stderr.write('WARNING: Multiple aliases matched "%s" in Any2ORF (returning first)\n' % name)
                return gene[0]['ORF']
            
        else:
            return gene['ORF']

    def printSummary(self):
        print '%s summary:' % self.__class__.__name__
        print ' Num. Genes = %d' % len(self.genes)
        print '(End)'

# Main test code
# geneIndex = GeneIndex() ; geneIndex.loadFile()
