# Logistic Regression module

from numarray import *
from numarray import random_array
import random

class LogReg_Hyp:
    """self.weights is linear weight vector, with bias at end (1,D+1)"""
    def __init__(self, inputDims = None):
        if inputDims is not None:
            self.weights = zeros([1,inputDims+1])
    def evaluate(self, input):
        """input is input vectors in rows (NxD)"""
        return (exp(- sum((input * self.weights[0,:-1]) + self.weights[0,-1], 1)) + 1) ** -1
        #net = (input * self.weights[0,:-1]) + self.weights[0,-1]
        #net = sum(net, 1)
        #sigmoid = (exp(- net) + 1) ** -1
        #return sigmoid

class LogReg_Alg:
    def learn(self, input, lrate = 0.01, momentum = 0.1, maxEpochs = 200, maxError = 0.001):
        """input is input vectors in rows, last col outputs (NxD+1)"""
        h = LogReg_Hyp()
        h.weights = random_array.normal(-0.01, 0.01, [1, input.shape[1]])
        deltaW = zeros([1, input.shape[1]])
        indices = range(0, input.shape[0])
        for epoch in range(0, maxEpochs):
            random.shuffle(indices)
            for t in indices:
                x = input[t,:-1]
                r = input[t,-1]
                y = h.evaluate(x)
                oldDeltaW = deltaW
                deltaW = lrate * (r - y) * concatenate([x, [1]]) + momentum * oldDeltaW
                h.weights += deltaW
        
        return h

def loadText(filename, sep = '\t'):
    return array([map(float, r.strip().split(sep)) for r in file(filename,'rt').readlines() if len(r.strip()) > 0])    

def test():
    iris = loadText('iris.dat.txt', sep=' ')
    iris[:,-1] = equal(iris[:,-1], 1)
    dtrain = iris[:100,:]
    dtest = iris[100:,:]
    alg = LogReg_Alg()
    hyp = alg.learn(dtrain)
    etrain = sum(equal(greater(hyp.evaluate(dtrain[:,:-1]), 0.5), dtrain[:,-1])) / float(dtrain.shape[0])
    etest  = sum(equal(greater(hyp.evaluate(dtest[:,:-1]), 0.5), dtest[:,-1])) / float(dtest.shape[0])
    print 'Prior probability = %f' % (sum(iris[:,-1]) / float(iris.shape[0]))
    print 'Training accuracy = %f' % etrain
    print 'Test accuracy = %f' % etest
    print 'weights = \n%s' % str(hyp.weights)
    
test()