from MicroarrayDataset import MicroarrayDataset

import os

class MicroarrayCollection:
    def __init__(self):
        #self.dataroot = 'C:\\Documents and Settings\\zbarutcu\\My Documents\\Machine Learning\\Olga\\Data\\Microarray\\'
        self.dataroot = os.getenv('HOME') + '/research/function/rawdata/Microarray/'

        #self.dataext = '.pcl.txt'
        self.dataext = '.knn.txt'
        print "*** WARNING: MicroarrayCollection using KNNimpute outputs."

        self.pickleExt = '.pickle'
        self.exportExt = '.mat.txt'

        #self.KNNimputeEXEPath = 'C:\\My Projects\\Cpp\\KNNimpute\\Release\\KNNimpute.exe'
        self.KNNimputeEXEPath = os.getenv('HOME') + '/research/function/external/KNNimpute/KNNimpute'

        self.datanames = [
            #'DeRisi', # small
            #'Zhu', # small
            #'Kuhn', # small
            'Chu',
            'Gasch2000',
            'Gasch2001',
            'Ogawa',
            'Shakoury-Elizeh',
            'Spellman_1',
            'Spellman_2',
            'Spellman_3',
            'Spellman_4',
            'Sudarsanam',
            'Yoshimoto'
            ]
        self.filenames = [self.dataroot + ds + self.dataext for ds in self.datanames]
        self.datasets = {}

    def exportMerged(self, outfile, ORFList, indexList = 'default', rowoffset = 0, coloffset = 0, missingStr = 'NaN'):
        """ indexList: None or [] -> don't save index column; default -> colORFList
            outfile: filename or file object
        """
        if indexList == 'default':
            indexList = ORFList
        elif indexList is None or indexList == []:
            indexList = None

        ftype = type(outfile)
        if ftype is str:
            outfile = file(outfile, 'wt')
        
        try:
            if indexList is not None:
                for (i, orf) in zip(range(0,len(ORFList)), ORFList):
                    outfile.write('%d\t%d\t%d\n' % (1+i+rowoffset, 1+coloffset, indexList.index(orf)))
                coloffset += 1
            
            for md in self.datasets.itervalues():
                md.export(outfile, ORFList, rowoffset = rowoffset, coloffset = coloffset, missingStr = missingStr)
                coloffset += md.getColCount()
        finally:
            if ftype is str:
                outfile.close()
        print 'Done.'
        return coloffset

    def exportAll(self, ORFList = None):
        print 'Exporting '
        for (dsname, md) in self.datasets.iteritems():
            print '%s...' % dsname,
            fname = self.dataroot + dsname + self.exportExt
            md.export(fname, ORFList)
        print '\nExported %d datasets.' % len(self.datasets)
        
    def loadAll(self, ORFList = None):
        print 'Loading '
        for dsname in self.datanames:
            print '%s...' % dsname,
            fname = self.dataroot + dsname + self.dataext
            md = MicroarrayDataset(ORFList)
            md.loadFile(fname)
            self.datasets[dsname] = md
        print '\nLoaded %d datasets.' % len(self.datasets)

    def pickleAll(self):
        print 'Pickling '
        import cPickle
        for (dsname, md) in self.datasets.iteritems():
            print '%s...' % dsname,
            fname = self.dataroot + dsname + self.pickleExt
            f = file(fname, "wb")
            try:
                cPickle.dump(md, f)
            finally:
                f.close()
        print '\nPickled %d datasets.' % len(self.datasets)

    def unpickleAll(self):
        print 'Unpickling '
        import cPickle
        for dsname in self.datanames:
            print '%s...' % dsname,
            fname = self.dataroot + dsname + self.pickleExt
            f = file(fname, "rb")
            try:
                self.datasets[dsname] = cPickle.load(f)
            finally:
                f.close()
        print '\nUnpickled %d datasets.' % len(self.datasets)

    def imputeAll(self, imputeFunc):
        print 'Imputing '
        for (dsname, md) in self.datasets.iteritems():
            print '%s...' % dsname,
            numImputed = imputeFunc(md)
            numTotal = md.getRowCount() * md.getColCount() 
            print '%d of %d cells imputed (%d%%)' % (numImputed, numTotal, int(100.0 * numImputed / numTotal + 0.5))
        print '\nImputed %d datasets.' % len(self.datasets)
        
    def impute_RowAvg(self, md):
        """Imputes given MicroarrayDataset in-place with row averages. Returns imputed cell count."""
        count = 0
        for row in md.rows:
            rowavg = None
            for (colname, colval) in row.items():
                if colname == 'ORF': continue
                if colval is None:
                    if rowavg is None:
                        # calculate row average, once if necessary
                        rowvalues = [float(v) for (n,v) in row.items() if n != 'ORF' and v is not None]
                        rowavg = str(sum(rowvalues) / len(rowvalues))
                    row[colname] = rowavg
                    count += 1
        return count
    
    def printSummary(self):
        print "%s instance:" % self.__class__.__name__
        print " Datasets:"
        totalCols = 0
        for (dsname, md) in self.datasets.iteritems():
            cols = md.getColCount()
            print "  %s: %d genes, %d columns" % (dsname, md.getRowCount(), cols)
            totalCols += cols            
        print " TOTAL: %d datasets, %d columns" % (len(self.datasets), totalCols)
        print "(End)"

    def runKNNimpute(self, k):
        import os
        print 'KNNimputing (k=%d)' % k
        for dsname in self.datanames:
            print '%s...' % dsname,
            fname = self.dataroot + dsname + self.dataext
            outfname = self.dataroot + dsname + '.knn.txt'
            retval = os.spawnl(os.P_WAIT, self.KNNimputeEXEPath, '"%s"' % self.KNNimputeEXEPath, '"%s"' % fname, `k`, '0', '"%s"' % outfname)
            print '(returned %d)' % retval
        print 'Done.'

    def getCommonORFs(self):
        from sets import Set
        if len(self.datasets) == 0:
            print 'No data loaded!'
            return Set()
        common = Set(self.datasets.values()[0].ORFs)
        for md in self.datasets.itervalues():
            common = common.intersection(Set(md.ORFs))
        return common

    def getTotalCols(self):
        return sum([md.getColCount() for md in self.datasets.values()])
