from TextData import TextData
from Table import Table
import sets

suppressWarnings = False

def WARNING(msg):
    if suppressWarnings: return
    print '*** WARNING: %s' % msg
    # raise Exception(msg)

class MicroarrayDataset(TextData):
    def __init__(self, ORFList = None):
        #self.DEFAULT_FILE_NAME = r"C:\Documents and Settings\Zafer\My Documents\Machine Learning\Olga\Data\Colocalization\complex_out_edit.txt"
        self.clear()
        self.ORFList = ORFList
        if ORFList is None:
            WARNING("MicroarrayDataset's ORFList not specified!")

    def clear(self):
        self.rows = Table(['ORF'])

    def loadLines(self, lines):
        print " parsing...",
        for line in lines:
            if line.startswith('EWEIGHT\t'): continue # skip
            if line.startswith('3XSSC\t'): continue # skip
            if line.startswith('YORF\t'):
                # load column headers
                labels = line.split('\t')[3:]
                self.rows.colNames.extend(labels)
                continue
            
            cols = line.split('\t')
            ORF = cols[0].strip()
            if self.ORFList is not None:
                if ORF not in self.ORFList:
                    # retry by removing first letter
                    gene = ORF
                    ORF = gene[1:]
                    if ORF not in self.ORFList:
                        WARNING('Gene lost in translation: %s' % gene)
                        continue
            
            fields = []
            for f in cols[3:]:
                f = f.strip()
                if f == '':
                    fields.append(None)
                else:
                    fields.append(f)
            
            self.rows.append([ORF] + fields)
        
        print " indexing..."
        self.rows.indexBy('ORF', True)
        #self.ORFs = sets.Set(self.rows.indexKeys('ORF'))
        self.ORFs = self.rows.indexKeys('ORF')
        print "OK"

    def export(self, outfile, ORFList = 'all', rowoffset = 0, coloffset = 0, missingStr = 'NaN'):
        """ ORFList: None or [] or 'all' -> use all self ORFs
        """
        if ORFList is None or ORFList == [] or ORFList == 'all':
            ORFList = self.ORFs
            WARNING('export: ORFList not specified; using all ORFs.')

        ftype = type(outfile)
        if ftype is str:
            f = file(outfile, 'wt')
        elif ftype is file:
            f = outfile
        
        try:
            # write items
            for (orf, r) in zip(ORFList, range(0, len(ORFList))):
                ind = self.rows.indexes['ORF']
                row = ind.get(orf) # may return None

                # write out the actual row
                for (colname, c) in zip(self.rows.colNames[1:], range(0, len(self.rows.colNames)-1)): # skip 'ORF', the first col
                    if row is None:
                        colvalue = missingStr
                    else:
                        colvalue = row[colname]                    
                        if colvalue is None:
                            colvalue = missingStr
                    
                    # write col
                    f.write('%d\t%d\t%s\n' % (r+1+rowoffset, c+1+coloffset, colvalue))
            
        finally:
            if ftype is str:
                f.close()

    def getRowCount(self):
        return len(self.rows)

    def getColCount(self):
        return len(self.rows.colNames) - 1

    def printSummary(self):
        print "%s instance:" % self.__class__.__name__
        print " Rows: %d" % self.getRowCount()
        print " Cols: %d" % self.getColCount()
        print "(End)"

#md = MicroarrayDataset.MicroarrayDataset() ; complex.loadFile()
#md = MicroarrayDataset.MicroarrayDataset(geneIndex.genes.indexKeys('ORF'))
#md.loadFile('C:\Documents and Settings\zbarutcu\My Documents\Machine Learning\Olga\Data\Microarray\Kuhn.pcl.txt')