"""
Transcription Factor Binding Sites data scraper for:
 http://www.ncbi.nlm.nih.gov/CBBresearch/Postdocs/Wataru/PROSPECT/data/tfdbs/transfactor.html

The output file consists of lines in the format:
 sitename TAB gene1 TAB gene2 TAB ... geneN CR LF

Zafer Barutcuoglu - 2003/12/27
"""

import urllib2

class TFScraper:
    def __init__(self):
        self.rootURL = 'http://www.ncbi.nlm.nih.gov/CBBresearch/Postdocs/Wataru/PROSPECT/data/tfdbs/transfactor.html'
        self.geneURL = 'http://cgsigma.cshl.org/cgi-bin/jz/getgene2?%s'
        self.TFSites = {} # siteName -> list of geneName

    def scrape(self):
        # download
        print 'Downloading list...',
        resp = urllib2.urlopen(self.rootURL)
        lines = resp.readlines()
        resp.close()
        print 'OK.'
        
        # parse
        for line in lines:
            if line.startswith('<a href='):
                i = line.index('"') + 1
                j = line.index('"', i)
                url = line[i:j]
                i = line.index('>', j) + 1
                j = line.index('<', i)
                tfsite = line[i:j]
                self.__scrapeTFSite(url, tfsite)
        
        print 'Done!'

    def __scrapeTFSite(self, url, tfsite):
        print 'Scraping %s...' % tfsite,
        # download
        resp = urllib2.urlopen(url, 'action=Get regulated genes')
        lines = resp.readlines()
        resp.close()
        
        # parse
        geneList = []
        for line in lines:
            if '<li>' in line:
                k = line.index('<li>') + 4
                i = line.index('">', k) + 2
                j = line.index('<', i)
                gene = line[i:j]
                orf = self.__scrapeORFName(gene)
                geneList.append(orf)
        
        # add tfsite
        self.TFSites[tfsite] = geneList
        print 'OK.'

    def __scrapeORFName(self, gene):
        resp = urllib2.urlopen(self.geneURL % gene)
        lines = resp.readlines()
        orfLinePrefix = '       ORF\t'
        for line in lines:
            if line.startswith(orfLinePrefix):
                return line[len(orfLinePrefix):].strip()
        print 'WARNING: ORF name not found for "%s"' % gene
        return gene

    def saveTFSites(self, path):
        outfile = file(path, 'wb')
        for (tfsite, genes) in self.TFSites.iteritems():
            outfile.write('%s\t%s\r\n' % (tfsite, '\t'.join(genes)))
        outfile.close()
