from TextData import TextData
from Table import Table
import sets
import os

suppressWarnings = False

def WARNING(msg):
    if suppressWarnings: return
    print '*** WARNING: %s' % msg
    # raise Exception(msg)

class TFSites(TextData):
    def __init__(self, minGeneCount = 2):
        #self.DEFAULT_FILE_NAME = r"C:\Documents and Settings\zbarutcu\My Documents\Machine Learning\Olga\Data\TransFactor\TFscraped.txt"
        self.DEFAULT_FILE_NAME = os.getenv('HOME') + "/research/function/rawdata/TransFactor/TFscraped.txt"
        
        self.minGeneCount = minGeneCount
        self.clear()

    def clear(self):
        self.sites = {}

    def loadLines(self, lines):
        print " parsing..."
        for line in lines:
            cols = [x.strip() for x in line.split('\t') if len(x.strip()) > 0]
            if len(cols)-1 < self.minGeneCount: continue
            self.sites[cols[0]] = cols[1:]
        
        print "Loaded %d TF sites." % self.getSiteCount()

    def export(self, outfile, ORFList, rowoffset = 0, coloffset = 0):
        ftype = type(outfile)
        if ftype is str:
            f = file(outfile, 'wt')
        elif ftype is file:
            f = outfile
        
        try:
            # write each row
            siteLists = self.sites.values()
            for (orf, r) in zip(ORFList, range(0, len(ORFList))):
                # write each col
                for (siteORFs, c) in zip(siteLists, range(0, len(siteLists))):
                    if orf in siteORFs:
                        f.write('%d\t%d\t1\n' % (r+1+rowoffset, c+1+coloffset))
            
            # ensure matrix size by forcing the last element
            if ORFList[-1] not in siteLists[-1]:
                f.write('%d\t%d\t0\n' % (len(ORFList)+rowoffset, len(siteLists)+coloffset))
            
        finally:
            if ftype is str:
                f.close()
        
        return len(siteLists)+coloffset

    def getSiteCount(self):
        return len(self.sites)

    def printSummary(self):
        print "%s instance:" % self.__class__.__name__
        print " Sites: %d" % self.getSiteCount()
        sizes = [len(x) for x in self.sites.values()]
        print " Min. genes: %d" % min(sizes)
        print " Max. genes: %d" % max(sizes)
        print "(End)"

#tfsites = TFSites.TFSites() ; tfsites.loadFile()
