"""
Zafer Barutcuoglu 2004/03/04
"""

import sys

def WARNING(msg):
    sys.stderr.write('*** WARNING: %s\n' % msg)
    # raise Exception(msg)

class Table(list):
    """
    Utility class like a DB table. Avoids storing dicts for every row.

    Data is internally stored as a list of dicts, with column indexes.
    """
    def __init__(self, colNames = []):
        self.colNames = colNames
        self.indexes = {}

    def indexBy(self, col, isUnique = False, ignore = [None, '']):
        D = {}
        if isUnique:
            for row in self:
                v = row[col]
                if v in ignore: continue
                if v in D: WARNING('Unique index "%s" violated: "%s"' % (col, v))
                D[v] = row
        else:
            for row in self:
                v = row[col]
                if v in ignoreElements: continue
                if v in D:
                    D[v].append(row)
                else:
                    D[v] = [row]
        self.indexes[col] = D
        
    def indexByList(self, col, isUnique = False, ignore = [None, ''], ignoreElements = [None, '']):
        D = {}
        if isUnique:
            for row in self:
                vs = row[col]
                if vs in ignore: continue
                for v in vs:
                    if v in ignoreElements: continue
                    if v in D: WARNING('Unique index "%s" violated: "%s"' % (col, v))
                    D[v] = row
        else:
            for row in self:
                vs = row[col]
                if vs in ignore: continue
                for v in vs:
                    if v in ignoreElements: continue
                    if v in D:
                        D[v].append(row)
                    else:
                        D[v] = [row]
        self.indexes[col] = D

    def deleteIndex(self, col):
##        if type(col) is int:
##            col = self.colNames.keys()[col]
        del self.indexes[col]

    def find(self, col, val):
##        if type(col) is str:
##            col = self.colNames.index(col)
        return self.indexes[col].get(val)

##    def col(self, colName):
##        return self.colNames.index(colName)

    def indexKeys(self, col):
##        if type(col) is str:
##            col = self.colNames.index(col)
        return self.indexes[col].keys()

    def append(self, row):
        if type(row) is dict:
            rowCols = row.keys()
            for colName in self.colNames:
                if colName not in rowCols: raise Exception('Table field missing: %s' % colName)
            list.append(self, row)
            return row
        elif type(row) is list:
            if len(row) != len(self.colNames): raise Exception('Field counts don\'t match: (%d <- %d)' % (len(self.colNames), len(row)))
            newRow = {}
            for (colName, val) in zip(self.colNames, row):
                newRow[colName] = val
            list.append(self, newRow)
            return newRow
        else:
            raise Exception('Invalid row type: %s' % str(type(row)))
