def WARNING(msg):
    print '*** WARNING: %s' % msg
    # raise Exception(msg)

class Table(list):
    """
    Utility class like a DB table. Avoids storing dicts for every row.

    Data is internally stored as a list of lists, with column indexes.
    """
    def __init__(self, colNames = []):
        self.colNames = colNames
        self.indexes = {}

    def indexBy(self, col, isUnique = False, ignore = [None, '']):
        colName = col
        if type(col) is str:
            col = self.colNames.index(col)
        D = {}
        if isUnique:
            for row in self:
                v = row[col]
                if v in ignore: continue
                if v in D: WARNING('Unique index "%s" violated: "%s"' % (colName, v))
                D[v] = row
        else:
            for row in self:
                v = row[col]
                if v in ignoreElements: continue
                if v in D:
                    D[v].append(row)
                else:
                    D[v] = [row]
        self.indexes[col] = D
        
    def indexByList(self, col, isUnique = False, ignore = [None, ''], ignoreElements = [None, '']):
        if type(col) is str:
            col = self.colNames.index(col)
        D = {}
        if isUnique:
            for row in self:
                vs = row[col]
                if vs in ignore: continue
                for v in vs:
                    if v in ignoreElements: continue
                    if v in D: WARNING('Unique index "%s" violated: "%s"' % (colName, v))
                    D[v] = row
        else:
            for row in self:
                vs = row[col]
                if vs in ignore: continue
                for v in vs:
                    if v in ignoreElements: continue
                    if v in D:
                        D[v].append(row)
                    else:
                        D[v] = [row]
        self.indexes[col] = D

    def deleteIndex(self, col):
        if type(col) is str:
            col = self.colNames.index(col)
        del self.indexes[col]

    def find(self, col, val):
        if type(col) is str:
            col = self.colNames.index(col)
        return self.indexes[col].get(val)

    def col(self, colName):
        return self.colNames.index(colName)

    def indexKeys(self, col):
        if type(col) is str:
            col = self.colNames.index(col)
        return self.indexes[col].keys()
