"""
Zafer Barutcuoglu 2004/03/04
"""

import sys

class TextData:
    """
    Parses text file of data lines..
    
    Required overloads:
        loadLines(...)

    Optional overloads:
        DEFAULT_FILE_NAME
        DEFAULT_URL

    Compression parameter values:
        None
        'gzip' or 'gz'
    """

    def loadFile(self, filename = 'default', compression = 'auto', bVerbose = True):
        if filename is None or filename == 'default':
            filename = self.DEFAULT_FILE_NAME
                
        if bVerbose: sys.stderr.write(' loading...'); sys.stderr.flush();
        f = file(filename, 'rb')
        if compression is not None: f = self.__decompress(f, compression, filename)
        lines = f.readlines()
        f.close()
        if bVerbose: sys.stderr.write('OK\n'); sys.stderr.flush();
        self.loadLines(lines)

    def loadURL(self, url = 'default', compression = 'auto', saveAs = None, bVerbose = True):
        if url is None or url == 'default':
            url = self.DEFAULT_URL
        import urllib2
        if bVerbose: sys.stderr.write(' downloading...'); sys.stderr.flush()
        resp = urllib2.urlopen(url)
        
        if compression is not None:
            bindata = resp.read()
            import StringIO
            strbuf = StringIO.StringIO(bindata)
            newbuf = self.__decompress(strbuf, compression, url)
            lines = newbuf.readlines()
            newbuf.close()
            strbuf.close()
            bindata = None # free mem
        else:
            lines = resp.readlines()
        
        resp.close()
        if bVerbose: sys.stderr.write('OK\n'); sys.stderr.flush();
        if saveAs is not None:
            if saveAs == 'default':
                saveAs = self.DEFAULT_FILE_NAME
            f = file(saveAs, 'wb')
            f.writelines(lines)
            f.close()
        self.loadLines(lines)
    
    def loadLines(self, lines):
        raise NotImplementedError

    def __decompress(self, f, compression, path):
        if compression == 'auto':
            i = path.rfind('.')
            if i >= 0:
                c = path[i+1:]
            else:
                c = None
        else:
            c = compression
            
        if c is None or c in ['none', 'None']:
            return f
        if c in ['gz','gzip']:
            import gzip
            return gzip.GzipFile(fileobj=f)
        
        if c == compression:
            raise Exception('Unknown compression mode %s' % str(compression))
        else:
            return f
