URL_ORF_SGD = 'http://db.yeastgenome.org/cgi-bin/locus.pl?locus=%s'
URL_GOID_GO = 'http://www.godatabase.org/cgi-bin/amigo/go.cgi?view=details&show_associations=direct&search_constraint=terms&depth=0&query=%s'

def exportAUCChangesToDOT(go, outfile, GOIDList, AUCChangesFile, bShortLabel = False):
    # read AUC Changes
    fintype = type(AUCChangesFile)
    if fintype is str:
        fin = file(AUCChangesFile, 'rt')
    elif ftype is file:
        fin = AUCChangesFile
        
    try:
        lines = fin.readlines()
    finally:
        if fintype is str:
            fin.close()

    dictTexts = {}
    dictColors = {}
    for line in lines:
        fields = line.split(' ')
        if len(fields) != 4:
            print "WARNING: Wrong number of fields in line:\n%s" % line
            continue
        goid = fields[0]
        AUC1 = float(fields[1])
        AUC2 = float(fields[2])
        change = float(fields[3])
        dictTexts[goid] = 'AUC1=%f\\nAUC2=%f\\ndiff=%f' % (AUC1, AUC2, change)
        
        maxChange =  0.20
        minChange = -0.20
        if change >= 0:
            if change > maxChange: change = maxChange
            level = int(255*(1.0-(change/maxChange)))
            hexcode = hex(level)[2:]
            hexcode = ('00'+hexcode)[-2:]
            rgbcode = '#%s%sFF' % (hexcode,hexcode)
        else:
            if change < minChange: change = minChange
            level = int(255*(1.0-(change/minChange)))
            hexcode = hex(level)[2:]
            hexcode = ('00'+hexcode)[-2:]
            rgbcode = '#FF%s%s' % (hexcode,hexcode)
        dictColors[goid] = rgbcode

    go.exportNodeParentsAsDOT(outfile, GOIDList, dictTexts, dictColors, None, bShortLabel)


def exportPredictionsToDOT(go, outfile, GOIDList, predictions, rowNum, bUseSigmoid = False, threshold = None):
    dictTexts = {}
    dictColors = {}
    for (pred, colNum) in zip(predictions[rowNum], range(0, len(GOIDList))):
        goid = GOIDList[colNum]
        
        if pred is not None:
            if bUseSigmoid:
                import math
                pred = 1/(1+math.exp(-pred))
            
            dictTexts[goid] = 'p = %f' % (pred)
            
            if threshold is not None:
                if pred > threshold:
                    pred = 1
                else:
                    pred = 0
            
            minPred = 0
            maxPred = 0.5
            if pred < minPred: pred = minPred
            if pred > maxPred: pred = maxPred
            level = int(255*(1.0-((pred-minPred)/(maxPred-minPred))))
            hexcode = hex(level)[2:]
            hexcode = ('00'+hexcode)[-2:]
            rgbcode = '#FF%s%s' % (hexcode,hexcode)
        else:
            rgbcode = '#AAAAAA'
        
        dictColors[goid] = rgbcode

    go.exportNodeParentsAsDOT(outfile, GOIDList, dictTexts, dictColors, None)


def exportAllPredictionsToDOT(go, ORFList, geneNumList, GOIDList, outdir, predictions, bUseSigmoid = False, threshold = None):
    for rowNum in range(0, len(predictions)):
        geneNum = geneNumList[rowNum]
        orf = ORFList[geneNum]
        outfile = outdir + ('/%s.dot' % (orf))
        exportPredictionsToDOT(go, outfile, GOIDList, predictions, rowNum, bUseSigmoid, threshold)


def saveWebSite(go, ORFList, geneNumList, GOIDList, outdir, predictions):
    def saveHomePage(go, ORFList, geneNumList, GOIDList, outdir):
        f = file(outdir + '/index.html', 'wt')
        try:
            f.write('<html>\n')
            f.write('<body>\n')
            
            f.write('(Scroll down for list by ORF)<br>\n')
            f.write('<h3>GO Nodes</h3>\n')
            
            SortedGOIDList = GOIDList[:]
            SortedGOIDList.sort()
            for goid in SortedGOIDList:
                nodename = go.nodesByID[goid].name
                f.write('<a href="%s.html">%s - %s</a> [<a href="%s">go</a>]<br>\n' % (goid.replace(':','_'), goid, nodename, URL_GOID_GO % goid))
            
            f.write('<h3>ORF Names</h3>\n')
            SortedORFList = [ORFList[geneNum] for geneNum in geneNumList]
            SortedORFList.sort()
            for orf in SortedORFList:
                f.write('<a href="%s.html">%s</a> [<a href="%s">sgd</a>]<br>\n' % (orf, orf, URL_ORF_SGD % orf))
            
            f.write('</body>\n')
            f.write('</html>\n')
        finally:
            f.close()

    def saveGenePage(go, ORFList, geneNumList, GOIDList, outdir, predictions, rowNum):
        geneNum = geneNumList[rowNum]
        orf = ORFList[geneNum]
        f = file(outdir + ('/%s.html' % (orf)), 'wt')
        try:
            f.write('<html>\n')
            f.write('<head><title>%s Predictions</title></head>\n' % (orf))
            f.write('<body>\n')
            
            f.write('<h3>%s</h3>\n' % (orf))
            f.write('View as Hierarchy (SVG): [<a target="_blank" href="svgz/raw/%s.dot.svgz">Raw</a>] [<a target="_blank" href="svgz/%s.dot.svgz">Bayes-corrected</a>]<p>\n' % (orf, orf))
            
            sortedPreds = zip(predictions[rowNum], GOIDList, range(0, len(GOIDList)))
            sortedPreds.sort(lambda x,y: -(x>y))
            for (pred, goid, colNum) in sortedPreds:
                nodename = go.nodesByID[goid].name
                f.write('%3.1f%% <a href="%s.html">%s - %s</a> [<a href="%s">go</a>]<br>\n' % (pred * 100, goid.replace(':','_'), goid, nodename, URL_GOID_GO % goid))
            
            f.write('</body>\n')
            f.write('</html>\n')
        finally:
            f.close()
    
    def saveGOIDPage(go, ORFList, geneNumList, GOIDList, outdir, predictions, colNum):
        goid = GOIDList[colNum]
        nodename = go.nodesByID[goid].name
        f = file(outdir + ('/%s.html' % (goid.replace(':','_'))), 'wt')
        try:
            f.write('<html>\n')
            f.write('<head><title>%s - %s Predictions</title></head>\n' % (goid, nodename))
            f.write('<body>\n')
            
            f.write('<h3>%s - %s</h3>\n' % (goid, nodename))
            
            sortedPreds = zip([row[colNum] for row in predictions], range(0, len(predictions)))
            sortedPreds.sort(lambda x,y: -(x>y))
            for (pred, rowNum) in sortedPreds:
                geneNum = geneNumList[rowNum]
                orf = ORFList[geneNum]
                f.write('%3.1f%% <a href="%s.html">%s</a> [<a href="%s">sgd</a>]<br>\n' % (pred * 100, orf, orf, URL_ORF_SGD % orf))
            
            f.write('</body>\n')
            f.write('</html>\n')
        finally:
            f.close()

    saveHomePage(go, ORFList, geneNumList, GOIDList, outdir)
    
    for rowNum in range(0, len(predictions)):
        saveGenePage(go, ORFList, geneNumList, GOIDList, outdir, predictions, rowNum)
    
    for colNum in range(0, len(GOIDList)):
        saveGOIDPage(go, ORFList, geneNumList, GOIDList, outdir, predictions, colNum)

def saveWebSiteData(go, ORFList, geneNumList, GOIDList, outdir, predictions):
    def saveIndex_GOID(go, ORFList, geneNumList, GOIDList, outdir):
        f = file(outdir + '/index_GOID.dat', 'wt')
        try:
            SortedGOIDList = GOIDList[:]
            SortedGOIDList.sort()
            for goid in SortedGOIDList:
                nodename = go.nodesByID[goid].name
                f.write(['"%s"' % goid, '"%s"' % nodename].join('|'))
        finally:
            f.close()

    def saveIndex_UnknownORF(go, ORFList, geneNumList, GOIDList, outdir):
        f = file(outdir + '/index_ORF.dat', 'wt')
        try:
            SortedORFList = [ORFList[geneNum] for geneNum in geneNumList]
            SortedORFList.sort()
            for orf in SortedORFList:
                f.write(['"%s"' % orf].join('|'))
        finally:
            f.close()

    def savePage_UnknownORF(go, ORFList, geneNumList, GOIDList, outdir, predictions, rowNum):
        geneNum = geneNumList[rowNum]
        orf = ORFList[geneNum]
        f = file(outdir + ('/Unknown_%s.dat' % (orf)), 'wt')
        try:
            sortedPreds = zip(predictions[rowNum], GOIDList, range(0, len(GOIDList)))
            sortedPreds.sort(lambda x,y: -(x>y))
            for (pred, goid, colNum) in sortedPreds:
                nodename = go.nodesByID[goid].name
                f.write(['"%s"' % goid, '"%s"' % nodename, pred].join('|'))
        finally:
            f.close()
    
    def savePage_GOID(go, ORFList, geneNumList, GOIDList, outdir, predictions, colNum):
        goid = GOIDList[colNum]
        nodename = go.nodesByID[goid].name
        f = file(outdir + ('/GOID_%s.dat' % (goid.replace(':','_'))), 'wt')
        try:
            sortedPreds = zip([row[colNum] for row in predictions], range(0, len(predictions)))
            sortedPreds.sort(lambda x,y: -(x>y))
            for (pred, rowNum) in sortedPreds:
                geneNum = geneNumList[rowNum]
                orf = ORFList[geneNum]
                f.write(['"%s"' % orf, pred].join('|'))
        finally:
            f.close()

    saveIndex_GOID(go, ORFList, geneNumList, GOIDList, outdir)
    saveIndex_UnknownORF(go, ORFList, geneNumList, GOIDList, outdir)
    
    for rowNum in range(0, len(predictions)):
        savePage_UnknownORF(go, ORFList, geneNumList, GOIDList, outdir, predictions, rowNum)
    
    for colNum in range(0, len(GOIDList)):
        savePage_GOID(go, ORFList, geneNumList, GOIDList, outdir, predictions, colNum)

def loadGeneNumListFromSparseText(filename):
    # geneNumList = analysis.loadGeneNumListFromSparseText(os.environ['HOME'] + '/research/function/data/ALL_unknowns.mat.txt')
    L = []
    for line in file(filename).readlines():
        fields = line.split('\t')
        if fields[1] == '1':
            L.append(int(fields[2]))
        else:
            break
    return L

def loadPredictionsFromMatlabText(filename):
    # predictions = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsUnknownRawBayesUp_withExcluded.mat.txt')
    def convert(s):
        s = s.strip()
        if s == 'NaN':
            t = None
        else:
            t = float(s)
        return t
    
    M = []
    for line in file(filename).readlines():
        M.append([convert(s) for s in line.split('  ') if len(s) > 0])
    return M

def loadORFList(filename):
    # ORFList = analysis.loadORFList(os.environ['HOME'] + '/research/function/data/geneNameIndex.txt')
    return [s.strip() for s in file(filename).readlines() if len(s.strip()) > 0]

def findNewAnnotations(go = None, assoc = None, goNew = None, assocNew = None, geneIndex = None, selectedGOIDs = None, ORFList = None, geneNumList = None):
    if geneIndex is None:
        import GeneIndex
        geneIndex = GeneIndex.GeneIndex()
        geneIndex.loadFile()
    if go is None:
        import ontology
        go = ontology.Ontology()
        go.loadFile()
    if goNew is None:
        import ontology
        goNew = ontology.Ontology()
        goNew.loadFile()
    if assoc is None:
        import association
        assoc = association.Association(translator = geneIndex.Any2ORF)
        assoc.loadFile()
        go.addAnnotations(assoc)
    if assocNew is None:
        import association
        assocNew = association.Association(translator = geneIndex.Any2ORF)
        assocNew.loadFile(os.environ['HOME'] + '/research/function/rawdata/GO/gene_association_20050705.sgd.gz')
        goNew.addAnnotations(assocNew)
    if selectedGOIDs is None:
        import SelectedGONodes
        selectedGOIDs = SelectedGONodes.NewGOIDs
    if ORFList is None:
        ORFList = loadORFList(os.environ['HOME'] + '/research/function/data/geneNameIndex.txt')
    if geneNumList is None:
        geneNumList = loadGeneNumListFromSparseText(os.environ['HOME'] + '/research/function/data/ALL_unknowns.mat.txt')

    from sets import Set
    newAnnotatedGenes = [g for g in assoc.genes if g['directNodes'] == ['GO:0000004'] and
        len(Set(assocNew.getNodesOfGene(g['ORF'])).intersection(selectedGOIDs)) > 0 and
        ORFList.index(g['ORF']) in geneNumList]
    newAnnotations = [(g['ORF'], n)
        for g in newAnnotatedGenes
            for n in Set(assocNew.getNodesOfGene(g['ORF'])).intersection(SelectedGONodes.NewGOIDs)]
    return newAnnotations

def predictNewAnnotations(newAnnotations, ORFList, geneNumList, GOIDList, predictions, threshold):
    ##predictionsSVM = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsUnknownRaw.mat.txt')
    ##predictionsBayes = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsUnknownRawBayesUp_withExcluded.mat.txt')
    ##newPredictionsSVM = analysis.predictNewAnnotations(newAnnotations, ORFList, geneNumList, SelectedGONodes.SelectedGOIDs, predictionsSVM, 0)
    ##allPairsForNewGenes = [(orf, goid) for orf in newAnnotatedGenes for goid in SelectedGONodes.NewGOIDs]
    ##for thr in [0.5, 0.90, 0.95, 0.99]:
    ##    newPredictionsBayes = analysis.predictNewAnnotations(newAnnotations, ORFList, geneNumList, SelectedGONodes.SelectedGOIDs, predictions, thr)
    ##    newDirectPredictionsBayes = analysis.predictNewAnnotations(newDirectAnnotations, ORFList, geneNumList, SelectedGONodes.SelectedGOIDs, predictions, thr)
    ##    print thr, sum(newPredictionsBayes), sum(newDirectPredictionsBayes), zip(newDirectPredictionsSVM, newDirectPredictionsBayes).count((0, 1)), zip(newDirectPredictionsSVM, newDirectPredictionsBayes).count((1, 0))
    ##
    L = []
    for (orf, goid) in newAnnotations:
        i = geneNumList.index(ORFList.index(orf))
        j = GOIDList.index(goid)
        pred = predictions[i][j]
        if pred is None:
            L.append(None)
        elif pred > threshold:
            L.append(1)
        else:
            L.append(0)
    return L


##Example script:
##
##import SelectedGONodes
##GOIDList = SelectedGONodes.SelectedGOIDs[:]
##
##geneNumList = analysis.loadGeneNumListFromSparseText(os.environ['HOME'] + '/research/function/data/ALL_unknowns.mat.txt')
##
##predictions = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsUnknownRawBayesUp.mat.txt')
##
##analysis.saveWebSite(go, ORFList, geneNumList, GOIDList, 'C:/temp/eraseme/genesite', predictions)

# Converting DOT to SVGZ and PS
##dot -Tsvgz AUCChanges_svml_lin_Cinf_RawUp_fill_105.dot -oAUCChanges_svml_lin_Cinf_RawUp_fill_105.svgz -Nheight=1 -Nwidth=1.5 -Nfixedsize=true -Nfontsize=30
##dot -Tps AUCChanges_svml_lin_Cinf_RawUp_fill_105.dot -oAUCChanges_svml_lin_Cinf_RawUp_fill_105.ps -Nheight=1 -Nwidth=1.5 -Nfixedsize=true -Nfontsize=30 -Gsize="7,7"

# Reindexing columns for NewGOIDs (GOIDList minus disconnected nodes)
##NewGOIDIndices = [GOIDList.index(s) for s in SelectedGONodes.NewGOIDs]
##NewPredictions = [[row[i] for i in NewGOIDIndices] for row in predictions]

# Drawing Label/Raw/Bayes SVGs for an interestingly corrected known ORF
##i = 427
##gene = ORFList[geneNumList[i]]
##analysis.exportPredictionsToDOT(go, 'J:/research/function/reports/%s_Raw.dot' % gene, NewGOIDs, NewPredictionsRaw, i, threshold = 0)
##analysis.exportPredictionsToDOT(go, 'J:/research/function/reports/%s_BayesUp.dot' % gene, NewGOIDs, NewPredictions, i, threshold = 0.5)
##analysis.exportPredictionsToDOT(go, 'J:/research/function/reports/%s_Labels.dot' % gene, NewGOIDs, NewLabels, i)
##>>> gene
##'YNL261W'

# Exporting HELD-OUT predictions
##ORFList = analysis.loadORFList(os.environ['HOME'] + '/research/function/data/geneNameIndex.txt')
##geneNumList = analysis.loadGeneNumListFromSparseText(os.environ['HOME'] + '/research/function/data/ALL_inputs.mat.txt')
##NewGOIDIndices = [GOIDList.index(s) for s in NewGOIDs]
##labelsWithExcluded = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/NewLabels.mat.txt')
##predictionsRawWithExcluded = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsRaw_withExcluded.mat.txt')
##predictionsBayesWithExcluded = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsRawBayes_withExcluded.mat.txt')
##NewPredictionsRawWithExcluded = [[row[i] for i in NewGOIDIndices] for row in predictionsRawWithExcluded]
##NewPredictionsBayesWithExcluded = [[row[i] for i in NewGOIDIndices] for row in predictionsBayesWithExcluded]
##NewLabelsWithExcluded = [[row[i] for i in NewGOIDIndices] for row in labelsWithExcluded]
##analysis.exportAllPredictionsToDOT(go, ORFList, geneNumList, SelectedGONodes.NewGOIDs, 'J:/research/function/reports/genesite/heldout/labels', NewLabelsWithExcluded)
##analysis.exportAllPredictionsToDOT(go, ORFList, geneNumList, SelectedGONodes.NewGOIDs, 'J:/research/function/reports/genesite/heldout/raw', NewPredictionsRawWithExcluded, bUseSigmoid = True)
##analysis.exportAllPredictionsToDOT(go, ORFList, geneNumList, SelectedGONodes.NewGOIDs, 'J:/research/function/reports/genesite/heldout/bayesup', NewPredictionsBayesWithExcluded)

# Exporting UNKNOWN predictions
##ORFList = analysis.loadORFList(os.environ['HOME'] + '/research/function/data/geneNameIndex.txt')
##geneNumList = analysis.loadGeneNumListFromSparseText(os.environ['HOME'] + '/research/function/data/ALL_inputs.mat.txt')
##NewGOIDIndices = [GOIDList.index(s) for s in SelectedGONodes.NewGOIDs]
##predictionsRawWithExcluded = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsUnknownRaw.mat.txt')
##predictionsBayesWithExcluded = analysis.loadPredictionsFromMatlabText(os.environ['HOME'] + '/research/function/rawresults/svml_lin_Cinf_OutputsUnknownRawBayesUp_withExcluded.mat.txt')
##NewPredictionsRawWithExcluded = [[row[i] for i in NewGOIDIndices] for row in predictionsRawWithExcluded]
##NewPredictionsBayesWithExcluded = [[row[i] for i in NewGOIDIndices] for row in predictionsBayesWithExcluded]
##analysis.exportAllPredictionsToDOT(go, ORFList, geneNumList, SelectedGONodes.NewGOIDs, 'J:/research/function/reports/genesite/unknown/raw', NewPredictionsRawWithExcluded, bUseSigmoid = True)
##analysis.exportAllPredictionsToDOT(go, ORFList, geneNumList, SelectedGONodes.NewGOIDs, 'J:/research/function/reports/genesite/unknown/bayesup', NewPredictionsBayesWithExcluded)
