# Telomere-related experiments on data from Chad

def saveIndicesUsingList(outfile, geneList, positiveList):
    """This does NOT use annotations at all, but outputs 1 for genes in ORFList, and 0 otherwise.
    """
    
    ftype = type(outfile)
    if ftype is str:
        f = file(outfile, 'wt')
    elif ftype is file:
        f = outfile
    
    try:
        # write items
        for (i, geneName) in zip(range(1,len(geneList)+1), geneList):
            if geneName in positiveList:
                f.write('%d\t%d\t1\n' % (i, 1))
        
        # ensure size by writing out item (m,n)
        lastGene = geneList[-1]
        if lastGene not in positiveList:
            f.write('%d\t%d\t0\n' % (len(geneList), 1))
    finally:
        if ftype is str:
            f.close()


# load gene index
import GeneIndex
geneIndex = GeneIndex.GeneIndex()
geneIndex.loadFile()

import os

# load data in groups of CGNs
dataDir = r'C:\Documents and Settings\Zafer\My Documents\Machine Learning\Olga\Data\Telomere'
groups = {}
for i in range(0,6):
    fileName = dataDir + os.sep + 'group' + `(i+1)` + '.txt'
    lines = file(fileName, 'rt').readlines()
    groups[i+1] = [geneIndex.Any2ORF(cgn.strip()) for cgn in lines]

levels = {}
levels[1] = groups[1]
levels[2] = groups[2]
levels[3] = groups[4]
levels[4] = groups[3] + groups[5]
levels[5] = groups[6]

cascades = {}
cascades[1] = levels[1]
cascades[2] = cascades[1] + levels[2]
cascades[3] = cascades[2] + levels[3]
cascades[4] = cascades[3] + levels[4]
cascades[5] = cascades[4] + levels[5]

import GRID
grid = GRID.Interactions(translator = geneIndex.Any2ORF)
grid.loadFile()
grid.printSummary()

import ontology
go = ontology.Ontology()
go.loadFile()
import association
assoc = association.Association(translator = geneIndex.Any2ORF)
assoc.loadFile()
go.addAnnotations(assoc)
go.printSummary()

from sets import Set
gridORFs = grid.genes
bpuORFs = [g['ORF'] for g in go.nodesByID['GO:0000004'].genes]
trainingORFs = [x for x in gridORFs if x not in bpuORFs]
bpuGRIDORFs = [x for x in gridORFs if x in bpuORFs]

for i in range(1,6):
    saveIndicesUsingList(r'C:\My Projects\Python\Bioinformatics\gridexp\telomereL%d_out.txt' % i, trainingORFs, levels[i])

print "Done."

##All 5 groups, 40 positives replicated to 2548 with 10% noise
##2728, 7349, 4444, 2851, 1889, 3211, 54,  7622, 906, 5751, 7653, 1389, 6450
##['YFL007W', 'YLR440C', 'YLR457C', 'YLL022C', 'YNL116W',
## 'YCR079W', 'YPL144W', 'YDR332W', 'YFL034W', 'YER139C',
## 'YDR071C', 'YPL176C', 'YLR352W']

##First 4 groups, 23 positives replicated to 3634 with 10% noise
##2728, 906
##['YFL007W', 'YFL034W']


